"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseMiniApp = void 0;
const PackagePath_1 = require("./PackagePath");
const packageJsonFileUtils_1 = require("./packageJsonFileUtils");
const tagoneline_1 = require("./tagoneline");
const PackageManager_1 = require("./PackageManager");
const utils = __importStar(require("./utils"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const log_1 = __importDefault(require("./log"));
const lodash_1 = __importDefault(require("lodash"));
const Platform_1 = __importDefault(require("./Platform"));
class BaseMiniApp {
    constructor({ miniAppPath, packagePath, }) {
        this.path = miniAppPath;
        const packageJsonPath = path_1.default.join(miniAppPath, 'package.json');
        if (!fs_1.default.existsSync(packageJsonPath)) {
            throw new Error(`This command should be run at the root of a mini-app`);
        }
        const packageJson = (0, packageJsonFileUtils_1.readPackageJsonSync)(miniAppPath);
        if (packageJson.ernPlatformVersion) {
            // TO REMOVE IN ERN 0.5.0
            log_1.default.warn(`
=================================================================
ernPlatformVersion will be deprecated soon
Please replace
  "ernPlatformVersion" : "${packageJson.ernPlatformVersion}"
with
  "ern" : { "version" : "${packageJson.ernPlatformVersion}" }
in the package.json of ${packageJson.name} MiniApp
=================================================================`);
        }
        else if (!packageJson.ern) {
            throw new Error((0, tagoneline_1.tagOneLine) `No ern section found in ${packageJson.name} package.json. Are you sure this is a MiniApp ?`);
        }
        this.packageJson = packageJson;
        this.packagePath = packagePath;
    }
    get name() {
        var _a, _b, _c, _d;
        return ((_d = (_b = (_a = this.packageJson.ern) === null || _a === void 0 ? void 0 : _a.miniAppName) !== null && _b !== void 0 ? _b : (_c = this.packageJson.ern) === null || _c === void 0 ? void 0 : _c.moduleName) !== null && _d !== void 0 ? _d : this.getUnscopedModuleName(this.packageJson.name));
    }
    getUnscopedModuleName(moduleName) {
        const npmScopeModuleRe = /(@.*)\/(.*)/;
        return npmScopeModuleRe.test(moduleName)
            ? npmScopeModuleRe.exec(moduleName)[2]
            : moduleName;
    }
    get normalizedName() {
        return this.name.replace(/-/g, '');
    }
    get pascalCaseName() {
        return `${this.normalizedName
            .charAt(0)
            .toUpperCase()}${this.normalizedName.slice(1)}`;
    }
    get scope() {
        const scopeCapture = /^@(.*)\//.exec(this.packageJson.name);
        if (scopeCapture) {
            return scopeCapture[1];
        }
    }
    get version() {
        return this.packageJson.version;
    }
    get platformVersion() {
        var _a, _b;
        return (_b = (_a = this.packageJson.ern) === null || _a === void 0 ? void 0 : _a.version) !== null && _b !== void 0 ? _b : this.packageJson.ernPlatformVersion;
    }
    get packageDescriptor() {
        return `${this.packageJson.name}@${this.packageJson.version}`;
    }
    get packageManager() {
        // Use ern.packageManager value from package.json if set
        const ernPackageManager = this.packageJson.ern.packageManager;
        if (ernPackageManager) {
            if (ernPackageManager === 'npm') {
                return PackageManager_1.PackageManager.npm();
            }
            else if (ernPackageManager === 'yarn') {
                return PackageManager_1.PackageManager.yarn();
            }
            else {
                throw new Error(`Invalid ern.packageManager value in package.json : ${ernPackageManager}.
Should be either yarn or npm`);
            }
        }
        // Otherwise favor yarn if it is installed
        else if (Platform_1.default.isYarnInstalled()) {
            return PackageManager_1.PackageManager.yarn();
        }
        // Finally fallback to npm
        else {
            return PackageManager_1.PackageManager.npm();
        }
    }
    async isPublishedToNpm() {
        return utils.isPublishedToNpm(PackagePath_1.PackagePath.fromString(`${this.packageJson.name}@${this.packageJson.version}`));
    }
    getPackageJsonDependencies() {
        return lodash_1.default.map(this.packageJson.dependencies, (val, key) => PackagePath_1.PackagePath.fromString(`${key}@${val}`));
    }
}
exports.BaseMiniApp = BaseMiniApp;
//# sourceMappingURL=BaseMiniApp.js.map