"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const code_push_1 = __importDefault(require("code-push"));
const log_1 = __importDefault(require("./log"));
class CodePushSdk {
    constructor(initConfig) {
        this.codePush = new code_push_1.default(initConfig.accessKey, initConfig.customHeaders, initConfig.customServerUrl, initConfig.proxy);
    }
    async releaseReact(appName, deploymentName, filePath, targetBinaryVersion, updateMetadata, disableDuplicateReleaseError) {
        if (updateMetadata.rollout === 100) {
            // If rollout is 100% we shouldn't pass it in the HTTP request
            delete updateMetadata.rollout;
        }
        try {
            const res = await this.codePush.release(appName, deploymentName, filePath, targetBinaryVersion, updateMetadata);
            return res;
        }
        catch (error) {
            if (disableDuplicateReleaseError &&
                error.statusCode === code_push_1.default.ERROR_CONFLICT) {
                log_1.default.warn(error.message);
            }
            else {
                throw new Error(JSON.stringify(error));
            }
        }
    }
    async promote(appName, sourceDeploymentName, destinationDeploymentName, updateMetadata, disableDuplicateReleaseError) {
        if (updateMetadata.rollout === 100) {
            // If rollout is 100% we shouldn't pass it in the HTTP request
            delete updateMetadata.rollout;
        }
        try {
            const res = await this.codePush.promote(appName, sourceDeploymentName, destinationDeploymentName, updateMetadata);
            return res;
        }
        catch (error) {
            if (disableDuplicateReleaseError &&
                error.statusCode === code_push_1.default.ERROR_CONFLICT) {
                log_1.default.warn(error.message);
            }
            else {
                throw new Error(JSON.stringify(error));
            }
        }
    }
    async patch(appName, deploymentName, updateMetadata) {
        if (updateMetadata.rollout === 100) {
            // If rollout is 100% we shouldn't pass it in the HTTP request
            delete updateMetadata.rollout;
        }
        return this.codePush.patchRelease(appName, deploymentName, updateMetadata.label, updateMetadata);
    }
}
exports.default = CodePushSdk;
//# sourceMappingURL=CodePushSdk.js.map