"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IosPluginConfigGenerator = void 0;
const path_1 = __importDefault(require("path"));
const fs_readdir_recursive_1 = __importDefault(require("fs-readdir-recursive"));
class IosPluginConfigGenerator {
    static fromPath(p) {
        return new IosPluginConfigGenerator(p);
    }
    constructor(p) {
        this.excludedDirectoriesRe = new RegExp(/sample|demo|example/i);
        this.root = p;
        this.files = (0, fs_readdir_recursive_1.default)(p, (x) => !this.excludedDirectoriesRe.test(x));
    }
    async generateConfig({ resolvePbxProjPath, }) {
        if (!this.doesPluginSupportIos) {
            throw new Error('No iOS project found');
        }
        const config = {};
        const pbxProjPath = this.pbxprojPaths.length > 1
            ? await resolvePbxProjPath(this.pbxprojPaths)
            : this.pbxprojPaths[0];
        const projName = this.getProjectName(pbxProjPath);
        const xcodeprojPath = path_1.default.dirname(pbxProjPath);
        const rootXcodeProjPath = path_1.default.dirname(xcodeprojPath);
        config.copy = [
            {
                dest: `{{{projectName}}}/Libraries/${projName}`,
                source: `${rootXcodeProjPath}/*`,
            },
        ];
        config.pbxproj = {
            addHeaderSearchPath: [
                `\"$(SRCROOT)/{{{projectName}}}/Libraries/${projName}/**\"`,
            ],
            addProject: [
                {
                    group: 'Libraries',
                    path: `${projName}/${projName}.xcodeproj`,
                    staticLibs: [
                        {
                            name: `lib${projName}.a`,
                            target: projName,
                        },
                    ],
                },
            ],
        };
        return config;
    }
    get doesPluginSupportIos() {
        return this.pbxprojPaths.length > 0;
    }
    getProjectName(pbxProjPath) {
        let match = pbxProjPath.match(/.+\/(.+)\.xcodeproj/);
        if (!match) {
            match = pbxProjPath.match(/^(.+)\.xcodeproj/);
        }
        return match[1];
    }
    get pbxprojPaths() {
        return this.files.filter((x) => x.endsWith('.pbxproj'));
    }
}
exports.IosPluginConfigGenerator = IosPluginConfigGenerator;
//# sourceMappingURL=IosPluginConfigGenerator.js.map