import { PackagePath } from './PackagePath';
export declare class LocalManifest {
    readonly manifestPath: string;
    constructor(manifestPath: string);
    getManifest(): Promise<any>;
    hasManifestId(manifestId: string): Promise<boolean>;
    getManifestData({ manifestId, platformVersion, }?: {
        manifestId?: string;
        platformVersion?: string;
    }): Promise<any | void>;
    /**
     * Return an array containing all top level plugin configuration directories
     * are matching an Electrode Native version lower or equal than the specified
     * maxVersion.
     *
     * For example, given the following directories :
     *
     *  /home/user/.ern/ern-override-manifest/plugins/ern_v0.5.0+
     *  /home/user/.ern/ern-override-manifest/plugins/ern_v0.10.0+
     *  /home/user/.ern/ern-override-manifest/plugins/ern_v0.13.0+
     *
     * And maxVersion='0.10.0'
     *
     * The function would return :
     *
     *  [
     *   '/home/user/.ern/ern-override-manifest/plugins/ern_v0.5.0+',
     *   '/home/user/.ern/ern-override-manifest/plugins/ern_v0.10.0+'
     *  ]
     * @param maxVersion The upper bound Electrode Native version.
     *                   Only plugin configuration directories lower than this
     *                   version will be returned.
     */
    getPluginsConfigurationDirectories(maxVersion?: string): string[];
    /**
     * Given a fixed versioned plugin, return the local file system path
     * to the directory containing the plugin configuration files, or
     * undefined if no configuration matching this plugin version exists
     * in the Manifest.
     * @param plugin Fixed versioned plugin for which to retrieve configuration
     * @param platformVersion Platform version that is querying for
     *                        the plugin configuration.
     */
    getPluginConfigurationPath(plugin: PackagePath, platformVersion?: string): Promise<string | void>;
    private getPluginConfigurationPathNew;
    /**
     * Old way of getting plugin configuration path
     *  -- To be deprecated --
     */
    private getPluginConfigurationPathLegacy;
}
