import { PackagePath } from './PackagePath';
import { NativeDependencies } from './nativeDependenciesLookup';
import { BaseMiniApp } from './BaseMiniApp';
export declare class MiniApp extends BaseMiniApp {
    static miniAppFsPathByPackagePath: Map<string, string>;
    static fromCurrentPath(): MiniApp;
    static fromPath(fsPath: string): MiniApp;
    static existInPath(p: string): boolean;
    static fromPackagePath(packagePath: PackagePath): Promise<MiniApp>;
    static create(miniAppName: string, packageName: string, { language, manifestId, packageManager, platformVersion, scope, skipInstall, template, }?: {
        language?: 'JavaScript' | 'TypeScript';
        manifestId?: string;
        packageManager?: 'npm' | 'yarn';
        platformVersion?: string;
        scope?: string;
        skipInstall?: boolean;
        template?: string;
    }): Promise<MiniApp>;
    constructor(miniAppPath: string, packagePath: PackagePath);
    getNativeDependencies({ manifestId, }?: {
        manifestId?: string;
    }): Promise<NativeDependencies>;
    getJsDependencies(): Promise<PackagePath[]>;
    addDependency(dependency: PackagePath, { dev, manifestId, peer, }?: {
        dev?: boolean;
        manifestId?: string;
        peer?: boolean;
    }): Promise<PackagePath | void>;
    /**
     * Perform checks to ensure that proper dependency version is picked based on manifest entry.
     *
     * @param dependency dependency to be added
     * @param manifestDependency dependency defined in manifest
     * @returns {Dependency} Dependency with proper version number
     */
    manifestConformingDependency(dependency: PackagePath, manifestDependency: PackagePath): PackagePath | void;
    upgrade({ manifestId, platformVersion, }?: {
        manifestId?: string;
        platformVersion?: string;
    }): Promise<any>;
    publishToNpm(): void;
    link(): Promise<void>;
    unlink(): Promise<void>;
    private addDevOrPeerDependency;
}
