"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MiniApp = void 0;
const child_process_1 = require("child_process");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const semver_1 = __importDefault(require("semver"));
const Manifest_1 = require("./Manifest");
const ModuleTypes_1 = require("./ModuleTypes");
const PackagePath_1 = require("./PackagePath");
const Platform_1 = __importDefault(require("./Platform"));
const clients_1 = require("./clients");
const config_1 = __importDefault(require("./config"));
const createTmpDir_1 = __importDefault(require("./createTmpDir"));
const log_1 = __importDefault(require("./log"));
const nativeDependenciesLookup_1 = require("./nativeDependenciesLookup");
const shell_1 = __importDefault(require("./shell"));
const utils = __importStar(require("./utils"));
const packageJsonFileUtils_1 = require("./packageJsonFileUtils");
const packageCache_1 = require("./packageCache");
const kax_1 = __importDefault(require("./kax"));
const BaseMiniApp_1 = require("./BaseMiniApp");
const lodash_1 = __importDefault(require("lodash"));
const getMetroBlacklistPath_1 = require("./getMetroBlacklistPath");
const npmIgnoreContent = `android/
ios/
.watchmanconfig
`;
const filesToRemove = [
    '.bundle/',
    'android/',
    'ios/',
    '.buckconfig',
    '.flowconfig',
    '.node-version',
    '.ruby-version',
    'Gemfile',
    'Gemfile.lock',
];
class MiniApp extends BaseMiniApp_1.BaseMiniApp {
    static fromCurrentPath() {
        return MiniApp.fromPath(process.cwd());
    }
    static fromPath(fsPath) {
        return new MiniApp(fsPath, PackagePath_1.PackagePath.fromString(fsPath));
    }
    static existInPath(p) {
        // Need to improve this one to check in the package.json if it contains the
        // ern object with miniapp type
        return fs_extra_1.default.pathExistsSync(path_1.default.join(p, 'package.json'));
    }
    static async fromPackagePath(packagePath) {
        let fsPackagePath;
        if (config_1.default.get('package-cache-enabled', true) &&
            !packagePath.isFilePath &&
            !(await utils.isGitBranch(packagePath))) {
            if (!(await packageCache_1.packageCache.isInCache(packagePath))) {
                fsPackagePath = await packageCache_1.packageCache.addToCache(packagePath);
            }
            else {
                fsPackagePath = await packageCache_1.packageCache.getObjectCachePath(packagePath);
            }
        }
        else {
            if (this.miniAppFsPathByPackagePath.has(packagePath.fullPath)) {
                fsPackagePath = this.miniAppFsPathByPackagePath.get(packagePath.fullPath);
            }
            else {
                fsPackagePath = (0, createTmpDir_1.default)();
                shell_1.default.pushd(fsPackagePath);
                try {
                    await clients_1.yarn.init();
                    await clients_1.yarn.add(packagePath);
                    const packageJson = await (0, packageJsonFileUtils_1.readPackageJson)('.');
                    const packageName = Object.keys(packageJson.dependencies)[0];
                    shell_1.default.rm(path_1.default.join(fsPackagePath, 'package.json'));
                    shell_1.default.mv(path_1.default.join(fsPackagePath, 'node_modules', packageName, '*'), fsPackagePath);
                    shell_1.default.rm('-rf', path_1.default.join(fsPackagePath, 'node_modules', packageName, '*'));
                }
                finally {
                    shell_1.default.popd();
                }
            }
        }
        if (!fsPackagePath) {
            throw new Error(`Could not resolve local path to ${packagePath}`);
        }
        this.miniAppFsPathByPackagePath.set(packagePath.fullPath, fsPackagePath);
        return new MiniApp(fsPackagePath, packagePath);
    }
    static async create(miniAppName, packageName, { language, manifestId, packageManager, platformVersion = Platform_1.default.currentVersion, scope, skipInstall, template, } = {}) {
        if (await fs_extra_1.default.pathExists(path_1.default.join('node_modules/react-native'))) {
            throw new Error('It seems like there is already a react native app in this directory. Use another directory.');
        }
        if (Platform_1.default.currentVersion !== platformVersion) {
            Platform_1.default.switchToVersion(platformVersion);
        }
        if (language && template) {
            log_1.default.warn('Both language and template are set. Ignoring language.');
        }
        let reactNativeVersion;
        const retrieveRnManifestTask = kax_1.default.task('Querying Manifest for react-native version to use');
        try {
            const reactNativeDependency = await Manifest_1.manifest.getNativeDependency(PackagePath_1.PackagePath.fromString('react-native'), {
                manifestId,
                platformVersion,
            });
            if (!reactNativeDependency) {
                throw new Error('react-native dependency is not defined in manifest. cannot infer version to be used');
            }
            reactNativeVersion = reactNativeDependency.version;
            if (!reactNativeVersion) {
                throw new Error('React Native version needs to be explicitly defined');
            }
            retrieveRnManifestTask.succeed(`Retrieved react-native version from Manifest [${reactNativeVersion}]`);
        }
        catch (e) {
            retrieveRnManifestTask.fail();
            throw e;
        }
        if (process.platform === 'darwin' &&
            semver_1.default.gte(reactNativeVersion, '0.60.0') &&
            !Platform_1.default.isCocoaPodsInstalled()) {
            throw new Error(`pod command not found.
CocoaPods is required starting from React Native 0.60 version.
You can find instructions to install CocoaPods @ https://cocoapods.org`);
        }
        // RN rejects standard kebab-case app/package names (most likely due to a
        // former restriction in an old version of Xcode). Until this is fixed in
        // RN, use a temporary RN-compatible name and rename the folder later
        const rnProjectName = /^[$A-Z_][0-9A-Z_$]*$/i.test(miniAppName)
            ? miniAppName
            : utils.camelize(miniAppName);
        const typescriptTemplate = semver_1.default.gte(reactNativeVersion, '0.60.0')
            ? 'react-native-template-typescript'
            : 'typescript';
        await kax_1.default
            .task(`Creating ${miniAppName} project using react-native@${reactNativeVersion}`)
            .run(clients_1.reactnative.init(rnProjectName, reactNativeVersion, {
            skipInstall,
            template: template
                ? template
                : language === 'TypeScript'
                    ? typescriptTemplate
                    : undefined,
        }));
        if (rnProjectName !== miniAppName) {
            fs_extra_1.default.renameSync(path_1.default.join(process.cwd(), rnProjectName), path_1.default.join(process.cwd(), miniAppName));
        }
        // Create .npmignore if it does not exist
        const npmIgnorePath = path_1.default.join(process.cwd(), miniAppName, '.npmignore');
        if (!fs_extra_1.default.existsSync(npmIgnorePath)) {
            await fs_extra_1.default.writeFile(npmIgnorePath, npmIgnoreContent);
        }
        // Inject ern specific data in MiniApp package.json
        const pathToMiniApp = path_1.default.join(process.cwd(), miniAppName);
        const appPackageJson = await (0, packageJsonFileUtils_1.readPackageJson)(pathToMiniApp);
        appPackageJson.ern = {
            moduleName: rnProjectName,
            moduleType: ModuleTypes_1.MINIAPP,
            packageManager: packageManager
                ? packageManager
                : Platform_1.default.isYarnInstalled()
                    ? 'yarn'
                    : 'npm',
            version: platformVersion,
        };
        appPackageJson.private = false;
        appPackageJson.keywords
            ? appPackageJson.keywords.push(ModuleTypes_1.MINIAPP)
            : (appPackageJson.keywords = [ModuleTypes_1.MINIAPP]);
        if (scope) {
            appPackageJson.name = `@${scope}/${packageName}`;
        }
        else {
            appPackageJson.name = packageName;
        }
        await (0, packageJsonFileUtils_1.writePackageJson)(pathToMiniApp, appPackageJson);
        const miniAppPath = path_1.default.join(process.cwd(), miniAppName);
        // For RN > 0.57
        // Write custom metro.config.js to make sure that packager does not crash
        // on Windows while building the Runner.
        // Do not write this file if creating a MiniApp from a project template
        // as the template might contain a custom metro.config.js file.
        // This is the responsibility of the template project owner to add
        // resolver blacklisted paths.
        // See https://github.com/facebook/react-native/issues/9136#issuecomment-348773574
        if (semver_1.default.gt(reactNativeVersion, '0.57.0') && !template) {
            await fs_extra_1.default.writeFile(path_1.default.join(miniAppPath, 'metro.config.js'), `const blacklist = require('${(0, getMetroBlacklistPath_1.getMetroBlacklistPath)(reactNativeVersion)}');
module.exports = {
  resolver: {
    blacklistRE: blacklist([
      // Ignore IntelliJ directories
      /.*\\.idea\\/.*/,
      // ignore git directories
      /.*\\.git\\/.*/,
      // Ignore android directories
      /.*\\/app\\/build\\/.*/,
    ]),
  },
  transformer: {
    getTransformOptions: async () => ({
      transform: {
        experimentalImportSupport: false,
        inlineRequires: false,
      },
    }),
  },
};
`);
        }
        // Remove react-native generated android and ios projects
        // They will be replaced with our owns when user uses `ern run android`
        // or `ern run ios` command
        // Also add ern-navigation dependency to the MiniApp.
        shell_1.default.pushd(miniAppPath);
        try {
            await kax_1.default.task('Removing unnecessary files from generated project').run(new Promise((resolve) => {
                filesToRemove.forEach((file) => shell_1.default.rm('-rf', file));
                resolve();
            }));
            const ernNavigationDependency = (await kax_1.default
                .task('Querying Manifest for ern-navigation version to use')
                .run(Manifest_1.manifest.getNativeDependency(PackagePath_1.PackagePath.fromString('ern-navigation'), {
                manifestId,
                platformVersion,
            }))) || PackagePath_1.PackagePath.fromString('ern-navigation');
            await kax_1.default
                .task(`Adding ${ernNavigationDependency} dependency`)
                .run(clients_1.yarn.add(ernNavigationDependency));
            return MiniApp.fromPath(miniAppPath);
        }
        finally {
            shell_1.default.popd();
        }
    }
    constructor(miniAppPath, packagePath) {
        super({ miniAppPath, packagePath });
    }
    async getNativeDependencies({ manifestId, } = {}) {
        return (0, nativeDependenciesLookup_1.findNativeDependencies)(path_1.default.join(this.path, 'node_modules'), {
            manifestId,
        });
    }
    // Return all javascript (non native) dependencies currently used by the MiniApp
    // This method checks dependencies from the package.json of the MiniApp and
    // exclude native dependencies (plugins).
    async getJsDependencies() {
        const nativeDependencies = await this.getNativeDependencies();
        const nativeDependenciesNames = lodash_1.default.map(nativeDependencies.all, (d) => d.name);
        const nativeAndJsDependencies = this.getPackageJsonDependencies();
        return lodash_1.default.filter(nativeAndJsDependencies, (d) => !nativeDependenciesNames.includes(`${d.name}@${d.version}`));
    }
    async addDependency(dependency, { dev, manifestId, peer, } = {}) {
        if (!dependency) {
            return log_1.default.error('dependency cannot be null');
        }
        if (dev || peer) {
            // Dependency is a devDependency or peerDependency
            // In that case we don't perform any checks at all (for now)
            await this.addDevOrPeerDependency(dependency, dev);
        }
        else {
            // Dependency is not a development dependency
            // In that case we need to perform additional checks and operations
            const basePathDependency = new PackagePath_1.PackagePath(dependency.basePath);
            const manifestNativeDependency = await Manifest_1.manifest.getNativeDependency(basePathDependency, { manifestId });
            const manifestDependency = manifestNativeDependency ||
                (await Manifest_1.manifest.getJsDependency(basePathDependency, { manifestId }));
            if (!manifestDependency) {
                // Dependency is not declared in manifest
                // We need to detect if this dependency is a pure JS one or if it's a native one or
                // if it contains transitive native dependencies
                const tmpPath = (0, createTmpDir_1.default)();
                process.chdir(tmpPath);
                await kax_1.default
                    .task(`${basePathDependency.toString()} is not declared in the manifest. Performing additional checks.`)
                    .run(this.packageManager.add(PackagePath_1.PackagePath.fromString(dependency.toString())));
                const nativeDependencies = await (0, nativeDependenciesLookup_1.findNativeDependencies)(path_1.default.join(tmpPath, 'node_modules'));
                if (lodash_1.default.isEmpty(nativeDependencies.all)) {
                    log_1.default.debug('Pure JS dependency');
                    // This is a pure JS dependency. Not much to do here -yet-
                }
                else if (nativeDependencies.all.length >= 1) {
                    log_1.default.debug(`One or more native dependencies identified: ${JSON.stringify(nativeDependencies.all)}`);
                    let dep;
                    for (dep of nativeDependencies.all) {
                        if (dependency.name === dep.name) {
                            if (await utils.isDependencyApiOrApiImpl(dep)) {
                                log_1.default.debug(`${dep.name} is an api or api-impl`);
                                log_1.default.warn(`${dep.name} is not declared in the Manifest. You might consider adding it.`);
                            }
                            else {
                                // This is a third party native dependency. If it's not in the master manifest,
                                // then it means that it is not supported by the platform yet. Fail.
                                throw new Error(`${dep.name} plugin is not yet supported. Consider adding support for it to the master manifest`);
                            }
                        }
                        else {
                            // This is a dependency which is not native itself but contains a native dependency as  transitive one (example 'native-base')
                            // If ern platform contains entry in the manifest but dependency versions do not align, report error
                            const manifestDep = await Manifest_1.manifest.getNativeDependency(new PackagePath_1.PackagePath(dep.basePath), { manifestId });
                            if (manifestDep) {
                                if (dep.version !== manifestDep.version) {
                                    throw new Error(`Version of transitive dependency ${dep.name}@${dep.version}
does not match version declared in manifest: ${manifestDep.version}`);
                                }
                            }
                        }
                    }
                }
            }
            else {
                if (dependency.version) {
                    log_1.default.debug(`Dependency:${dependency.name} defined in manifest, performing version match`);
                    // If the dependency & manifest version differ, log error and exit
                    if (dependency.version !== manifestDependency.version) {
                        throw new Error(`${dependency.name} was not added to the MiniApp`);
                    }
                }
            }
            // Checks have passed add the dependency
            process.chdir(this.path);
            await kax_1.default
                .task(`Adding ${manifestDependency
                ? manifestDependency.toString()
                : dependency.toString()} to ${this.name}`)
                .run(this.packageManager.add(manifestDependency || PackagePath_1.PackagePath.fromString(dependency.toString())));
            return manifestDependency ? manifestDependency : dependency;
        }
    }
    /**
     * Perform checks to ensure that proper dependency version is picked based on manifest entry.
     *
     * @param dependency dependency to be added
     * @param manifestDependency dependency defined in manifest
     * @returns {Dependency} Dependency with proper version number
     */
    manifestConformingDependency(dependency, manifestDependency) {
        if (!dependency.version ||
            dependency.version === manifestDependency.version) {
            // If no version was specified for this dependency, we're good, just use the version
            // declared in the manifest
            return manifestDependency;
        }
        else {
            // Dependency version mismatch. Let the user know of potential impacts and suggest user to
            // updat the version in the manifest
            // TODO : If not API/API impl, we need to ensure that plugin is supported by platform
            // for the provided plugin version
            log_1.default.warn(`${dependency.toString()} version mismatch.`);
            log_1.default.warn(`Manifest version: ${manifestDependency.version || 'undefined'}`);
            log_1.default.warn(`Wanted version: ${dependency.version || 'undefined'}`);
            log_1.default.warn(`You might want to update the version in your Manifest to add this dependency to ${this.name}`);
            return dependency;
        }
    }
    async upgrade({ manifestId, platformVersion = Platform_1.default.currentVersion, } = {}) {
        // Update all modules versions in package.json
        const manifestDependencies = await Manifest_1.manifest.getJsAndNativeDependencies({
            manifestId,
            platformVersion,
        });
        for (const manifestDependency of manifestDependencies) {
            if (this.packageJson.dependencies[manifestDependency.basePath]) {
                const dependencyManifestVersion = manifestDependency.version;
                const localDependencyVersion = this.packageJson.dependencies[manifestDependency.basePath];
                if (dependencyManifestVersion !== localDependencyVersion) {
                    log_1.default.info(`${manifestDependency.basePath} : ${localDependencyVersion} => ${dependencyManifestVersion}`);
                    this.packageJson.dependencies[manifestDependency.basePath] =
                        dependencyManifestVersion;
                }
            }
        }
        // Update ernPlatformVersion in package.json
        if (!this.packageJson.ern) {
            throw new Error(`In order to upgrade, please first replace "ernPlatformVersion" : "${this.packageJson.ernPlatformVersion}" in your package.json
with "ern" : { "version" : "${this.packageJson.ernPlatformVersion}" } instead`);
        }
        this.packageJson.ern.version = platformVersion;
        // Write back package.json
        const appPackageJsonPath = path_1.default.join(this.path, 'package.json');
        await fs_extra_1.default.writeFile(appPackageJsonPath, JSON.stringify(this.packageJson, null, 2));
        process.chdir(this.path);
        await kax_1.default.task('Running yarn install').run(this.packageManager.install());
    }
    publishToNpm() {
        (0, child_process_1.execSync)(`npm publish --prefix ${this.path}`);
    }
    async link() {
        const miniAppsLinks = config_1.default.get('miniAppsLinks', {});
        const previousLinkPath = miniAppsLinks[this.packageJson.name];
        if (previousLinkPath && previousLinkPath !== this.path) {
            log_1.default.warn(`Replacing previous link [${this.packageJson.name} => ${previousLinkPath}]`);
        }
        else if (previousLinkPath && previousLinkPath === this.path) {
            return log_1.default.warn(`Link is already created for ${this.packageJson.name} with same path`);
        }
        miniAppsLinks[this.packageJson.name] = this.path;
        config_1.default.set('miniAppsLinks', miniAppsLinks);
        log_1.default.info(`${this.packageJson.name} link created [${this.packageJson.name} => ${this.path}]`);
    }
    async unlink() {
        const miniAppsLinks = config_1.default.get('miniAppsLinks', {});
        if (miniAppsLinks[this.packageJson.name]) {
            delete miniAppsLinks[this.packageJson.name];
            config_1.default.set('miniAppsLinks', miniAppsLinks);
            log_1.default.info(`${this.packageJson.name} link was removed`);
        }
        else {
            return log_1.default.warn(`No link exists for ${this.packageJson.name}`);
        }
    }
    async addDevOrPeerDependency(dependency, dev) {
        const depPath = PackagePath_1.PackagePath.fromString(dependency.toString());
        if (dev) {
            await kax_1.default
                .task(`Adding ${dependency.toString()} to MiniApp devDependencies`)
                .run(this.packageManager.add(depPath, { dev: true }));
        }
        else {
            await kax_1.default
                .task(`Adding ${dependency.toString()} to MiniApp peerDependencies`)
                .run(this.packageManager.add(depPath, { peer: true }));
        }
    }
}
exports.MiniApp = MiniApp;
// Session cache
MiniApp.miniAppFsPathByPackagePath = new Map();
//# sourceMappingURL=MiniApp.js.map