"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PackageManager = void 0;
const NpmCli_1 = require("./NpmCli");
const YarnCli_1 = require("./YarnCli");
class PackageManager {
    static npm() {
        return new PackageManager('npm');
    }
    static yarn() {
        return new PackageManager('yarn');
    }
    constructor(type) {
        this.type = type;
        this.yarnCli = new YarnCli_1.YarnCli();
        this.npmCli = new NpmCli_1.NpmCli();
    }
    async add(dependencyPath, { dev, peer, } = {}) {
        return this.type === 'npm'
            ? this.npmCli.install({ dependencyPath, dev, peer })
            : this.yarnCli.add(dependencyPath, { dev, peer });
    }
    async init() {
        return this.type === 'npm' ? this.npmCli.init() : this.yarnCli.init();
    }
    async install() {
        return this.type === 'npm' ? this.npmCli.install() : this.yarnCli.install();
    }
    async upgrade(dependencyPath) {
        return this.type === 'npm'
            ? this.npmCli.update(dependencyPath)
            : this.yarnCli.upgrade(dependencyPath);
    }
}
exports.PackageManager = PackageManager;
//# sourceMappingURL=PackageManager.js.map