export declare class PackagePath {
    /**
     * Creates a PackagePath from a string
     * @param path Full path to the package (registry|git|file)
     */
    static fromString(path: string): PackagePath;
    /**
     * Full path to the package
     */
    readonly fullPath: string;
    /**
     * Package path without version
     * - File path        : path without scheme prefix
     * - Git path         : path without branch/tag/commit
     * - Registry path    : package name (including scope if any)
     */
    readonly basePath: string;
    /**
     * Version of the package
     * - File path        : version from package.json
     * - Git path         : branch/tag/commit || undefined
     * - Descriptor path  : package version || undefined
     */
    readonly version?: string;
    /**
     * Name of the package
     * - File path        : name from package.json
     * - Git path         : undefined
     * - Registry path    : package name
     */
    readonly name?: string;
    /**
     * Creates a PackagePath
     * @param path Full path to the package (registry|git|file)
     */
    constructor(path: string);
    get isGitPath(): boolean;
    get isFilePath(): boolean;
    get isRegistryPath(): boolean;
    same(other: PackagePath, { ignoreVersion, }?: {
        ignoreVersion?: boolean;
    }): boolean;
    toString(): string;
}
