"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginConfigGenerator = void 0;
const log_1 = __importDefault(require("./log"));
const AndroidPluginConfigGenerator_1 = require("./AndroidPluginConfigGenerator");
const IosPluginConfigGenerator_1 = require("./IosPluginConfigGenerator");
class PluginConfigGenerator {
    static async generateFromPath({ pluginPath, revolveBuildGradlePath, resolvePbxProjPath, resolveDependencyVersion, }) {
        const result = {};
        const androidGenerator = AndroidPluginConfigGenerator_1.AndroidPluginConfigGenerator.fromPath(pluginPath);
        const iosGenerator = IosPluginConfigGenerator_1.IosPluginConfigGenerator.fromPath(pluginPath);
        if (androidGenerator.doesPluginSupportAndroid) {
            try {
                result.pluginConfig = result.pluginConfig || {};
                result.pluginConfig.android = await androidGenerator.generateConfig({
                    resolveDependencyVersion,
                    revolveBuildGradlePath,
                });
                result.androidPluginSource =
                    await androidGenerator.generatePluginSource();
                log_1.default.info('Generated Android plugin configuration');
            }
            catch (e) {
                throw new Error(`Failed to generate Android Plugin Configuration.

Reason: ${e.message}`);
            }
        }
        else {
            log_1.default.warn(`[Skipping Android generation] Looks like this plugin does not support Android.`);
        }
        if (iosGenerator.doesPluginSupportIos) {
            try {
                result.pluginConfig = result.pluginConfig || {};
                result.pluginConfig.ios = await iosGenerator.generateConfig({
                    resolvePbxProjPath,
                });
                log_1.default.info('Generated iOS plugin configuration');
            }
            catch (e) {
                throw new Error(`Failed to generate iOS Plugin Configuration.

Reason: ${e.message}`);
            }
        }
        else {
            log_1.default.warn(`[Skipping iOS generation] Looks like this plugin does not support iOS.`);
        }
        return result;
    }
}
exports.PluginConfigGenerator = PluginConfigGenerator;
//# sourceMappingURL=PluginConfigGenerator.js.map