import { PackagePath } from './PackagePath';
export interface PackagePathWithResolvedVersion {
    pkgPath: PackagePath;
    version: string;
}
export declare class YarnLockParser {
    /**
     * Creates an instance of YarnLockParser given a
     * path to a yarn.lock file
     * @param yarnLockPath Path the to a yarn.lock file
     */
    static fromPath(yarnLockPath: string): YarnLockParser;
    /**
     * Creates an instance of YarnLockParser given the
     * content of a yarn.lock file as a string
     * @param yarnLockContent yarn.lock file content
     */
    static fromContent(yarnLockContent: string): YarnLockParser;
    /**
     * The yarn.lock file content
     */
    readonly content: string;
    /**
     * The parsed yarn.lock
     */
    readonly parsed: any;
    private constructor();
    /**
     * Find a given package in the yarn.lock
     * @param pkg A package path with or without version
     * @returns An array of PackagePathWithResolvedVersion containing all
     * packages matching the given PackagePath, along with their resolved
     * version.
     */
    findPackage(pkg: PackagePath): PackagePathWithResolvedVersion[];
    /**
     * Finds all packages which have a given dependency
     * @param dependency The dependency as a PackagePath
     * @returns An array of PackagePathWithResolvedVersion containing all
     * packages that are dependent on the given dependency, along with
     * their resolved version.
     */
    findPackagesWithDependency(dependency: PackagePath): PackagePathWithResolvedVersion[];
    /**
     * Builds a dependency tree for a given dependency.
     * @param dep The dependency for which to build a tree
     * @param level Used for recursion, should be left to default
     * @param obj Used for recursion, should be left to default
     * @param topLevel Used for recursion, should be left to default
     * @returns The dependency tree as an object matching
     * treeify specs https://github.com/notatestuser/treeify
     */
    buildDependencyTree(dep: PackagePath, level?: string, obj?: any, topLevel?: boolean): any;
}
