"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.YarnLockParser = void 0;
const lockfile = __importStar(require("@yarnpkg/lockfile"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const PackagePath_1 = require("./PackagePath");
class YarnLockParser {
    /**
     * Creates an instance of YarnLockParser given a
     * path to a yarn.lock file
     * @param yarnLockPath Path the to a yarn.lock file
     */
    static fromPath(yarnLockPath) {
        if (!fs_extra_1.default.pathExistsSync(yarnLockPath)) {
            throw new Error(`Path to yarn.lock ${yarnLockPath} does not exist`);
        }
        return new YarnLockParser(fs_extra_1.default.readFileSync(yarnLockPath, 'utf8'));
    }
    /**
     * Creates an instance of YarnLockParser given the
     * content of a yarn.lock file as a string
     * @param yarnLockContent yarn.lock file content
     */
    static fromContent(yarnLockContent) {
        return new YarnLockParser(yarnLockContent);
    }
    constructor(yarnLockContent) {
        this.content = yarnLockContent;
        this.parsed = lockfile.parse(yarnLockContent);
    }
    /**
     * Find a given package in the yarn.lock
     * @param pkg A package path with or without version
     * @returns An array of PackagePathWithResolvedVersion containing all
     * packages matching the given PackagePath, along with their resolved
     * version.
     */
    findPackage(pkg) {
        return pkg.version
            ? Object.keys(this.parsed.object)
                .filter((k) => k === pkg.fullPath)
                .map((k) => {
                var _a;
                return ({
                    pkgPath: PackagePath_1.PackagePath.fromString(k),
                    version: (_a = this.parsed.object[k]) === null || _a === void 0 ? void 0 : _a.version,
                });
            })
            : Object.keys(this.parsed.object)
                .filter((k) => k.startsWith(`${pkg.basePath}@`))
                .map((k) => {
                var _a;
                return ({
                    pkgPath: PackagePath_1.PackagePath.fromString(k),
                    version: (_a = this.parsed.object[k]) === null || _a === void 0 ? void 0 : _a.version,
                });
            });
    }
    /**
     * Finds all packages which have a given dependency
     * @param dependency The dependency as a PackagePath
     * @returns An array of PackagePathWithResolvedVersion containing all
     * packages that are dependent on the given dependency, along with
     * their resolved version.
     */
    findPackagesWithDependency(dependency) {
        return Object.entries(this.parsed.object)
            .filter(([key, value]) => Object.entries(value.dependencies || []).some(([k, v]) => k === dependency.basePath && v === dependency.version))
            .map(([k, v]) => {
            var _a;
            return ({
                pkgPath: PackagePath_1.PackagePath.fromString(k),
                version: (_a = this.parsed.object[k]) === null || _a === void 0 ? void 0 : _a.version,
            });
        });
    }
    /**
     * Builds a dependency tree for a given dependency.
     * @param dep The dependency for which to build a tree
     * @param level Used for recursion, should be left to default
     * @param obj Used for recursion, should be left to default
     * @param topLevel Used for recursion, should be left to default
     * @returns The dependency tree as an object matching
     * treeify specs https://github.com/notatestuser/treeify
     */
    buildDependencyTree(dep, level = 'top', obj = {}, topLevel = true) {
        if (level === 'top') {
            const pkgs = this.findPackage(dep);
            if (pkgs) {
                for (const pkg of pkgs) {
                    let o;
                    if (topLevel) {
                        const objKey = `${pkg.pkgPath.fullPath} [${pkg.version}]`;
                        obj[objKey] = {};
                        o = obj[objKey];
                    }
                    this.buildDependencyTree(pkg.pkgPath, 'inside', o || obj, false);
                }
            }
        }
        else {
            const pkgs = this.findPackagesWithDependency(dep);
            if (pkgs) {
                for (const pkg of pkgs) {
                    const objKey = `${pkg.pkgPath.fullPath} [${pkg.version}]`;
                    obj[objKey] = {};
                    const o = obj[objKey];
                    this.buildDependencyTree(pkg.pkgPath, 'top', o, false);
                }
            }
        }
        return obj;
    }
}
exports.YarnLockParser = YarnLockParser;
//# sourceMappingURL=YarnLockParser.js.map