"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultJSCVersion = exports.getDefaultHermesVersion = exports.androidEmulatorPath = exports.androidAdbPath = exports.androidSdkRoot = exports.installApk = exports.getDevices = exports.getAndroidAvds = exports.launchAndroidActivityDetached = exports.launchAndroidActivity = exports.getGradleByPlatform = exports.buildAndInstallApp = exports.androidGetBootAnimProp = exports.waitForAndroidDevice = exports.installAndLaunchApp = exports.runAndroidUsingAvdImage = exports.askUserToSelectAvdEmulator = exports.runAndroid = exports.runAndroidApk = exports.runAndroidProject = exports.resolveAndroidVersions = exports.DEFAULT_TARGET_COMPATIBILITY = exports.DEFAULT_SOURCE_COMPATIBILITY = exports.DEFAULT_TARGET_SDK_VERSION = exports.DEFAULT_SUPPORT_LIBRARY_VERSION = exports.DEFAULT_MIN_SDK_VERSION_POST_RN64 = exports.DEFAULT_MIN_SDK_VERSION_PRE_RN64 = exports.DEFAULT_KOTLIN_VERSION = exports.DEFAULT_JSC_VARIANT = exports.DEFAULT_GRADLE_DISTRIBUTION_VERSION = exports.DEFAULT_COMPILE_SDK_VERSION = exports.DEFAULT_BUILD_TOOLS_VERSION = exports.DEFAULT_ANDROIDX_LIFECYCLE_EXTENSIONS_VERSION = exports.DEFAULT_ANDROIDX_APPCOMPACT_VERSION = exports.DEFAULT_ANDROID_GRADLE_PLUGIN_VERSION = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const inquirer_1 = __importDefault(require("inquirer"));
const shell_1 = __importDefault(require("./shell"));
const config_1 = __importDefault(require("./config"));
const deviceConfigUtil = __importStar(require("./deviceConfig"));
const log_1 = __importDefault(require("./log"));
const childProcess_1 = require("./childProcess");
const kax_1 = __importDefault(require("./kax"));
const semver_1 = __importDefault(require("semver"));
// ==============================================================================
// Default value for android build config
// ==============================================================================
exports.DEFAULT_ANDROID_GRADLE_PLUGIN_VERSION = '7.0.4';
exports.DEFAULT_ANDROIDX_APPCOMPACT_VERSION = '1.1.0';
exports.DEFAULT_ANDROIDX_LIFECYCLE_EXTENSIONS_VERSION = '2.1.0';
exports.DEFAULT_BUILD_TOOLS_VERSION = '31.0.0';
exports.DEFAULT_COMPILE_SDK_VERSION = '31';
exports.DEFAULT_GRADLE_DISTRIBUTION_VERSION = '7.3.3';
exports.DEFAULT_JSC_VARIANT = 'android-jsc';
exports.DEFAULT_KOTLIN_VERSION = '1.6.10';
exports.DEFAULT_MIN_SDK_VERSION_PRE_RN64 = '19';
exports.DEFAULT_MIN_SDK_VERSION_POST_RN64 = '21';
exports.DEFAULT_SUPPORT_LIBRARY_VERSION = '28.0.0';
exports.DEFAULT_TARGET_SDK_VERSION = '30';
exports.DEFAULT_SOURCE_COMPATIBILITY = 'VERSION_1_8';
exports.DEFAULT_TARGET_COMPATIBILITY = 'VERSION_1_8';
const ANDROID_DEVICE_INFO = `
https://developer.android.com/studio/run/emulator-commandline
https://developer.android.com/studio/run/emulator`;
function resolveAndroidVersions({ androidGradlePlugin = exports.DEFAULT_ANDROID_GRADLE_PLUGIN_VERSION, androidxAppcompactVersion = exports.DEFAULT_ANDROIDX_APPCOMPACT_VERSION, androidxLifecycleExtrnsionsVersion = exports.DEFAULT_ANDROIDX_LIFECYCLE_EXTENSIONS_VERSION, buildToolsVersion = exports.DEFAULT_BUILD_TOOLS_VERSION, compileSdkVersion = exports.DEFAULT_COMPILE_SDK_VERSION, gradleDistributionVersion = exports.DEFAULT_GRADLE_DISTRIBUTION_VERSION, kotlinVersion = exports.DEFAULT_KOTLIN_VERSION, minSdkVersion, reactNativeAarVersion, sourceCompatibility = exports.DEFAULT_SOURCE_COMPATIBILITY, supportLibraryVersion = exports.DEFAULT_SUPPORT_LIBRARY_VERSION, targetCompatibility = exports.DEFAULT_TARGET_COMPATIBILITY, targetSdkVersion = exports.DEFAULT_TARGET_SDK_VERSION, reactNativeVersion, } = {}) {
    const resolvedMinSdkVersion = minSdkVersion
        ? minSdkVersion
        : semver_1.default.gte(reactNativeVersion, '0.64.0')
            ? exports.DEFAULT_MIN_SDK_VERSION_POST_RN64
            : exports.DEFAULT_MIN_SDK_VERSION_PRE_RN64;
    reactNativeAarVersion = reactNativeAarVersion !== null && reactNativeAarVersion !== void 0 ? reactNativeAarVersion : reactNativeVersion;
    return {
        androidGradlePlugin,
        androidxAppcompactVersion,
        androidxLifecycleExtrnsionsVersion,
        buildToolsVersion,
        compileSdkVersion,
        gradleDistributionVersion,
        kotlinVersion,
        minSdkVersion: resolvedMinSdkVersion,
        reactNativeAarVersion,
        sourceCompatibility,
        supportLibraryVersion,
        targetCompatibility,
        targetSdkVersion,
    };
}
exports.resolveAndroidVersions = resolveAndroidVersions;
// ==============================================================================
// Misc utilities
// ==============================================================================
// Returns a promise that will get resolved after a given delay (in ms)
async function delay(ms) {
    return new Promise((resolve, reject) => {
        setTimeout(() => {
            resolve();
        }, ms);
    });
}
// Returns a promise that will get rejected after a given timeout (in ms)
async function failAfterTimeOut(ms) {
    return new Promise((resolve, reject) => {
        setTimeout(() => {
            reject(new Error(`Failed to collect the device/emulator state.${ANDROID_DEVICE_INFO}`));
        }, ms);
    });
}
// ==============================================================================
// Core android stuff
// ==============================================================================
//
// Build and run a project on an Android emulator or connected device
// The `devDebug` variant will be built and launched on an emulator selected by
// the user (this function prompts the user with a list of available avd to choose from)
//
// Assumptions :
// - devDebug variant exists in the project
// Params :
// - activityName : The name of the Activity to start (default "MainActivity")
// - launchFlags: Flags to pass to the application when launching it
// - packageName : Name of the package containing the application
// Options :
// - projectPath : Absolute or relative path to the root of the Android projectPath
async function runAndroidProject({ activityName = 'MainActivity', launchFlags, packageName, projectPath, }) {
    return runAndroid({
        activityName,
        launchFlags,
        packageName,
        projectPath,
    });
}
exports.runAndroidProject = runAndroidProject;
//
// Run an android APK on an Android emulator or connected device
//
// Params :
// - apkPath : Absolute or relative path to the APK
// - packageName : Name of the package containing the application
// Options :
// - activityName : The name of the Activity to start (default "MainActivity")
async function runAndroidApk({ activityName = 'MainActivity', apkPath, launchFlags, packageName, }) {
    return runAndroid({
        activityName,
        apkPath,
        launchFlags,
        packageName,
    });
}
exports.runAndroidApk = runAndroidApk;
async function runAndroid({ activityName, apkPath, launchFlags, packageName, projectPath, }) {
    const devices = await getDevices();
    // install and launch the app if 1 emulator instance is running
    if (devices.length === 1) {
        log_1.default.debug(`${devices[0].split('\t')[0]} is running ...`);
        await installAndLaunchApp({
            activityName,
            apkPath,
            launchFlags,
            packageName,
            projectPath,
        });
    }
    else if (devices.length > 1) {
        throw new Error('More than one device/emulator is running !');
    }
    else {
        const avdImageName = await askUserToSelectAvdEmulator();
        return runAndroidUsingAvdImage({
            activityName,
            apkPath,
            avdImageName,
            launchFlags,
            packageName,
            projectPath,
        });
    }
}
exports.runAndroid = runAndroid;
async function askUserToSelectAvdEmulator() {
    const avdImageNames = await getAndroidAvds();
    if (Array.isArray(avdImageNames) && avdImageNames.length === 0) {
        throw new Error(`No Emulator or device found. Launch an emulator manually or connect a device.${ANDROID_DEVICE_INFO}`);
    }
    const deviceConfig = config_1.default.get(deviceConfigUtil.ANDROID_DEVICE_CONFIG);
    // Check if user has set the usePreviousEmulator flag to true
    if (avdImageNames && deviceConfig) {
        if (deviceConfig.usePreviousDevice) {
            // Get the name of previously used device
            const deviceId = deviceConfig.deviceId;
            // Check if avd image still exists
            const avdIndex = avdImageNames.indexOf(deviceId);
            if (avdIndex > -1) {
                return `${avdImageNames[avdIndex]}`;
            }
        }
    }
    // if avd image is still not resolved
    const { avdImageName } = await inquirer_1.default.prompt([
        {
            choices: avdImageNames,
            message: 'Choose Android device image',
            name: 'avdImageName',
            type: 'list',
        },
    ]);
    // Update the device Config
    if (deviceConfig) {
        deviceConfig.deviceId = avdImageName;
        config_1.default.set(deviceConfigUtil.ANDROID_DEVICE_CONFIG, deviceConfig);
    }
    return `${avdImageName}`;
}
exports.askUserToSelectAvdEmulator = askUserToSelectAvdEmulator;
// Does the job of actually running the app
// It orchestrates a few tasks to actually get the job done
// Params :
// - projectPath : Absolute or relative path to the root of the Android projectPath
// - packageName : name of the package containing the application
// - avdImageName : name of the avd image to use (device image)
async function runAndroidUsingAvdImage({ activityName, apkPath, launchFlags, avdImageName, packageName, projectPath, }) {
    // https://issuetracker.google.com/issues/37137213
    (0, childProcess_1.spawnp)(androidEmulatorPath(), ['-avd', avdImageName], { detached: true });
    await kax_1.default
        .task('Waiting for device to start')
        .run(Promise.race([waitForAndroidDevice(), failAfterTimeOut(3 * 60 * 1000)]));
    await installAndLaunchApp({
        activityName,
        apkPath,
        launchFlags,
        packageName,
        projectPath,
    });
}
exports.runAndroidUsingAvdImage = runAndroidUsingAvdImage;
// Does the job of installing and running the app
// It orchestrates a few tasks to actually get the job done
// Params :
// - projectPath : Absolute or relative path to the root of the Android projectPath
// - packageName : name of the package containing the application
async function installAndLaunchApp({ activityName, apkPath, launchFlags, packageName, projectPath, }) {
    if (projectPath) {
        await kax_1.default
            .task('Building and installing application')
            .run(buildAndInstallApp(projectPath));
    }
    else if (apkPath) {
        await kax_1.default.task('Installing APK').run(installApk(apkPath));
    }
    await kax_1.default.task('Launching Android Application').run(Promise.resolve());
    launchAndroidActivityDetached(packageName, activityName, { launchFlags });
}
exports.installAndLaunchApp = installAndLaunchApp;
// Utility method that basically completes whenever the android device is ready
// It check device readiness every 2 sec (poll way)
async function waitForAndroidDevice() {
    let androidBootAnimProp = await androidGetBootAnimProp();
    while (!androidBootAnimProp.toString().startsWith('stopped')) {
        await delay(2000);
        androidBootAnimProp = await androidGetBootAnimProp();
    }
}
exports.waitForAndroidDevice = waitForAndroidDevice;
// Utility method to know when the prop init.svc.bootanim is there
// which indicates somehow that device is ready to install APK and such
async function androidGetBootAnimProp() {
    return (0, childProcess_1.execp)(`${androidAdbPath()} wait-for-device shell getprop init.svc.bootanim`);
}
exports.androidGetBootAnimProp = androidGetBootAnimProp;
// Build & install application on the device
// params :
// - projectPath : Absolute or relative path to the root of the Android project
// containing the application
async function buildAndInstallApp(projectPath) {
    shell_1.default.pushd(projectPath);
    try {
        const gradlew = getGradleByPlatform();
        return (0, childProcess_1.execp)(`${gradlew} installDebug`);
    }
    finally {
        shell_1.default.popd();
    }
}
exports.buildAndInstallApp = buildAndInstallApp;
function getGradleByPlatform() {
    return /^win/.test(process.platform) ? 'gradlew' : './gradlew';
}
exports.getGradleByPlatform = getGradleByPlatform;
// Utility method to launch a specific activity from a given package
// Params :
// - packageName : name of the package containing the application
// - activityName : name of the Activity to launch
async function launchAndroidActivity(packageName, activityName) {
    return (0, childProcess_1.execp)(`${androidAdbPath()} shell am start -n ${packageName}/.${activityName}`);
}
exports.launchAndroidActivity = launchAndroidActivity;
// Utility method to launch a specific activity from a given packager
// Will spawn the command (detached mode)
function launchAndroidActivityDetached(packageName, activityName, { launchFlags = '' } = {}) {
    (0, childProcess_1.spawnp)(androidAdbPath(), [
        'shell',
        'am',
        'start',
        '-n',
        `${packageName}/.${activityName}`,
        launchFlags,
    ], { detached: true });
}
exports.launchAndroidActivityDetached = launchAndroidActivityDetached;
// Utility method to list all available android avd images (emulator images)
async function getAndroidAvds() {
    const stdout = await (0, childProcess_1.execp)(`${androidEmulatorPath()} -list-avds`);
    let avdArr = [];
    if (stdout) {
        avdArr = stdout.toString().trim().split(/\r?\n/);
    }
    return avdArr;
}
exports.getAndroidAvds = getAndroidAvds;
// Utility method to query what device instances are connected to the adb server
async function getDevices() {
    const stdout = await (0, childProcess_1.execp)(`${androidAdbPath()} devices`);
    /*
      stdout for running command  $adb devices
      List of devices attached
      * daemon not running. starting it now at tcp:5037 *
      * daemon started successfully *
    */
    const stdOutArr = stdout.toString().trim().split(/\r?\n/);
    // remove stdout 'List of devices attached' (position 0)
    // and remove stdout related to daemon
    return stdOutArr.filter((entry, i) => i > 0 && !entry.includes('* daemon'));
}
exports.getDevices = getDevices;
async function installApk(pathToApk) {
    return (0, childProcess_1.execp)(`${androidAdbPath()} install -r ${pathToApk}`);
}
exports.installApk = installApk;
function androidSdkRoot() {
    var _a;
    return (_a = process.env.ANDROID_SDK_ROOT) !== null && _a !== void 0 ? _a : process.env.ANDROID_HOME;
}
exports.androidSdkRoot = androidSdkRoot;
function androidAdbPath() {
    const sdkRoot = androidSdkRoot();
    return sdkRoot ? `${sdkRoot}/platform-tools/adb` : 'adb';
}
exports.androidAdbPath = androidAdbPath;
function androidEmulatorPath() {
    const sdkRoot = androidSdkRoot();
    if (sdkRoot) {
        if (fs_extra_1.default.existsSync(`${sdkRoot}/emulator/emulator`)) {
            return `${sdkRoot}/emulator/emulator`;
        }
        if (fs_extra_1.default.existsSync(`${sdkRoot}/tools/emulator`)) {
            return `${sdkRoot}/tools/emulator`;
        }
    }
    return 'emulator';
}
exports.androidEmulatorPath = androidEmulatorPath;
/**
 * Returns the default Hermes engine (hermes-engine) package version used by a given React Native version.
 * Only works for versions of RN >= 0.60.0 as hermes-engine package was introduced in this version.
 */
function getDefaultHermesVersion(reactNativeVersion) {
    if (semver_1.default.gte(reactNativeVersion, '0.66.0')) {
        // https://github.com/facebook/react-native/blob/v0.66.0/package.json#L98
        return '~0.9.0';
    }
    else if (semver_1.default.gte(reactNativeVersion, '0.65.0')) {
        // https://github.com/facebook/react-native/blob/v0.65.0/package.json#L98
        return '~0.8.1';
    }
    else if (semver_1.default.gte(reactNativeVersion, '0.64.0')) {
        // https://github.com/facebook/react-native/blob/v0.64.0/package.json#L97
        return '~0.7.0';
    }
    else if (semver_1.default.gte(reactNativeVersion, '0.63.0')) {
        // https://github.com/facebook/react-native/blob/v0.63.0/package.json#L98
        return '~0.5.0';
    }
    else if (semver_1.default.gte(reactNativeVersion, '0.62.0')) {
        return '~0.4.0';
    }
    else if (semver_1.default.gte(reactNativeVersion, '0.60.0')) {
        return '^0.2.1';
    }
    else {
        throw new Error('This function can only be called for versions of React Native >= 0.60.0');
    }
}
exports.getDefaultHermesVersion = getDefaultHermesVersion;
/**
 * Returns the default JavaScriptCore engine version used
 * by a given React Native version.
 * Only works for versions of RN >= 0.60.0 as dynamic jsc-android
 * package was introduced in this version.
 */
function getDefaultJSCVersion(reactNativeVersion) {
    if (semver_1.default.gte(reactNativeVersion, '0.65.0')) {
        return '^250230.2.1';
    }
    else if (semver_1.default.gte(reactNativeVersion, '0.61.0')) {
        return '^245459.0.0';
    }
    else if (semver_1.default.gte(reactNativeVersion, '0.60.0')) {
        return '245459.0.0';
    }
    else {
        throw new Error('This function can only be called for versions of React Native >= 0.60.0');
    }
}
exports.getDefaultJSCVersion = getDefaultJSCVersion;
//# sourceMappingURL=android.js.map