"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.spawnp = exports.execp = exports.promisifyChildProcess = void 0;
const child_process_1 = require("child_process");
const cross_spawn_1 = require("cross-spawn");
const log_1 = __importDefault(require("./log"));
function promisifyChildProcess(child) {
    return new Promise((resolve, reject) => {
        child.addListener('error', (err) => reject(err));
        child.addListener('exit', (code) => {
            if (code === 0) {
                resolve();
            }
            else {
                reject(new Error(`ChildProcess exited with code ${code}`));
            }
        });
    });
}
exports.promisifyChildProcess = promisifyChildProcess;
async function execp(command, options, loggers = {
    stderr: log_1.default.debug.bind(log_1.default),
    stdout: log_1.default.trace.bind(log_1.default),
}) {
    log_1.default.trace(`execp => command: ${command} options: ${JSON.stringify(options)}`);
    return new Promise((resolve, reject) => {
        const cp = (0, child_process_1.exec)(command, options, (error, stdout, stderr) => {
            if (error) {
                reject(error);
            }
            else {
                resolve(stdout.toString());
            }
        });
        if (cp.stdout) {
            cp.stdout.on('data', (data) => loggers.stdout(data.toString()));
        }
        if (cp.stderr) {
            cp.stderr.on('data', (data) => loggers.stderr(data.toString()));
        }
    });
}
exports.execp = execp;
async function spawnp(command, args = [], options = {}, loggers = {
    stderr: log_1.default.debug.bind(log_1.default),
    stdout: log_1.default.trace.bind(log_1.default),
}) {
    log_1.default.trace(`spawnp => command: ${command} args: ${JSON.stringify(args)} options: ${JSON.stringify(options)}`);
    const cp = (0, cross_spawn_1.spawn)(command, args, options);
    cp.stdout.on('data', (data) => loggers.stdout(data.toString()));
    cp.stderr.on('data', (data) => loggers.stderr(data.toString()));
    return promisifyChildProcess(cp);
}
exports.spawnp = spawnp;
//# sourceMappingURL=childProcess.js.map