"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InMemErnConfig = void 0;
class InMemErnConfig {
    constructor(config = {}, ernRcDir) {
        this.config = config;
        this.ernRcDir = ernRcDir;
        this.obj = config;
    }
    get(key, defaultValue) {
        const transObj = this.envTransform(this.obj);
        return key
            ? transObj[key] !== undefined
                ? transObj[key]
                : defaultValue
            : JSON.parse(JSON.stringify(transObj));
    }
    set(key, value) {
        this.obj[key] = value;
    }
    del(key) {
        if (key && this.obj.hasOwnProperty(key)) {
            delete this.obj[key];
            return true;
        }
        return false;
    }
    envTransform(obj) {
        let objStr = JSON.stringify(obj);
        objStr = objStr
            // Replace all ${env.[ENV_VAR_KEY]} with resolved values
            .replace(/\$\{env\.([^\}]+)\}/g, (match, envKey) => process.env[envKey])
            // Replace ${PWD} with current process working directory
            .replace(/\$\{PWD\}/g, process.cwd().replace(/\\/g, '\\\\'))
            // Replace ${ERNRC} with directory containing the .ernrc config
            .replace(/\$\{ERNRC\}/g, this.ernRcDir.replace(/\\/g, '\\\\'));
        return JSON.parse(objStr);
    }
}
exports.InMemErnConfig = InMemErnConfig;
//# sourceMappingURL=InMemErnConfig.js.map