"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonFileErnConfig = void 0;
const InMemErnConfig_1 = require("./InMemErnConfig");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
class JsonFileErnConfig {
    constructor(jsonFilePath) {
        this.jsonFilePath = jsonFilePath;
        const conf = fs_1.default.existsSync(this.jsonFilePath)
            ? JSON.parse(fs_1.default.readFileSync(this.jsonFilePath, 'utf-8'))
            : {};
        this.inMemConfig = new InMemErnConfig_1.InMemErnConfig(conf, path_1.default.dirname(jsonFilePath));
    }
    get(key, defaultValue) {
        return this.inMemConfig.get(key, defaultValue);
    }
    set(key, value) {
        this.inMemConfig.set(key, value);
        this.persist();
    }
    del(key) {
        const hasDeletedKey = this.inMemConfig.del(key);
        if (hasDeletedKey) {
            this.persist();
        }
        return hasDeletedKey;
    }
    persist() {
        fs_1.default.writeFileSync(this.jsonFilePath, JSON.stringify(this.inMemConfig.get(), null, 2));
    }
}
exports.JsonFileErnConfig = JsonFileErnConfig;
//# sourceMappingURL=JsonFileErnConfig.js.map