"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProxyAgentFromErnConfig = exports.createProxyAgentFromUrl = void 0;
const config_1 = __importDefault(require("./config"));
const tunnel_1 = __importDefault(require("tunnel"));
const url_1 = __importDefault(require("url"));
function createProxyAgentFromUrl(proxyUrl, { https } = {}) {
    const supportedProtocols = ['http:', 'https:'];
    const protocol = proxyUrl.protocol;
    if (!supportedProtocols.includes(protocol)) {
        throw new Error(`Only http and https protocols are supported (protocol: ${protocol})`);
    }
    if (!proxyUrl.port) {
        throw new Error(`port is missing from proxy url ${proxyUrl.toString()}`);
    }
    const proxy = {
        host: proxyUrl.hostname,
        port: parseInt(proxyUrl.port, 10),
    };
    if (https) {
        return protocol === 'http:'
            ? { agent: tunnel_1.default.httpsOverHttp({ proxy }), isHttpsAgent: false }
            : { agent: tunnel_1.default.httpsOverHttps({ proxy }), isHttpsAgent: true };
    }
    else {
        return protocol === 'http:'
            ? { agent: tunnel_1.default.httpOverHttp({ proxy }), isHttpsAgent: false }
            : { agent: tunnel_1.default.httpOverHttps({ proxy }), isHttpsAgent: true };
    }
}
exports.createProxyAgentFromUrl = createProxyAgentFromUrl;
function createProxyAgentFromErnConfig(configKey, { https } = {}) {
    const proxyUrl = config_1.default.get(configKey);
    if (proxyUrl) {
        return createProxyAgentFromUrl(new url_1.default.URL(proxyUrl), { https });
    }
    else if (process.env.http_proxy) {
        return createProxyAgentFromUrl(new url_1.default.URL(process.env.http_proxy), {
            https,
        });
    }
    else if (process.env.https_proxy) {
        return createProxyAgentFromUrl(new url_1.default.URL(process.env.https_proxy), {
            https,
        });
    }
}
exports.createProxyAgentFromErnConfig = createProxyAgentFromErnConfig;
//# sourceMappingURL=createProxyAgent.js.map