"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createZippedBundle = void 0;
const yazl_1 = __importDefault(require("yazl"));
const path_1 = __importDefault(require("path"));
const fs_readdir_recursive_1 = __importDefault(require("fs-readdir-recursive"));
const log_1 = __importDefault(require("./log"));
async function createZippedBundle({ bundlePath, assetsPath, }) {
    log_1.default.debug('Creating zipped bundle');
    const zippedbundle = new yazl_1.default.ZipFile();
    log_1.default.trace(`Adding ${bundlePath} to zipped bundle`);
    zippedbundle.addFile(bundlePath, path_1.default.basename(bundlePath));
    const assetsFiles = (0, fs_readdir_recursive_1.default)(assetsPath);
    for (const assetFile of assetsFiles) {
        const pathToAssetFile = path_1.default.join(assetsPath, assetFile);
        log_1.default.trace(`Adding ${pathToAssetFile} to zipped bundle`);
        zippedbundle.addFile(pathToAssetFile, assetFile);
    }
    zippedbundle.end();
    const chunks = [];
    zippedbundle.outputStream.on('data', (chunk) => {
        chunks.push(chunk);
    });
    return new Promise((resolve, reject) => {
        zippedbundle.outputStream.on('error', (err) => {
            reject(err);
        });
        zippedbundle.outputStream.on('end', () => {
            log_1.default.debug('Bundle was successfully zipped');
            resolve(Buffer.concat(chunks));
        });
    });
}
exports.createZippedBundle = createZippedBundle;
//# sourceMappingURL=createZippedBundle.js.map