"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppPlatformDescriptor = void 0;
const NativePlatform_1 = require("../NativePlatform");
const AppNameDescriptor_1 = require("./AppNameDescriptor");
class AppPlatformDescriptor {
    static fromString(s) {
        const [name, platform] = s.split(':');
        if (!platform) {
            throw new Error(`Invalid AppPlatformDescriptor literal ${s}.
An AppPlatformDescriptor literal must be formatted as <appPlatform>:<appPlatform>.`);
        }
        if (!NativePlatform_1.supportedNativePlatforms.includes(platform)) {
            throw new Error(`${platform} is not a supported platform.
Supported platforms : ${NativePlatform_1.supportedNativePlatforms.join(' || ')}`);
        }
        return new AppPlatformDescriptor(name, platform);
    }
    constructor(name, platform) {
        this.name = name;
        this.platform = platform;
    }
    toAppNameDescriptor() {
        return new AppNameDescriptor_1.AppNameDescriptor(this.name);
    }
    toAppPlatformDescriptor() {
        return this;
    }
    toString() {
        return `${this.name}:${this.platform}`;
    }
}
exports.AppPlatformDescriptor = AppPlatformDescriptor;
//# sourceMappingURL=AppPlatformDescriptor.js.map