"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppVersionDescriptor = void 0;
const NativePlatform_1 = require("../NativePlatform");
const AppPlatformDescriptor_1 = require("./AppPlatformDescriptor");
const AppNameDescriptor_1 = require("./AppNameDescriptor");
class AppVersionDescriptor {
    static fromString(s) {
        const [name, platform, version] = s.split(':');
        if (!version || !platform) {
            throw new Error(`Invalid AppVersionDescriptor literal ${s}.
An AppVersionDescriptor literal must be formatted as <appName>:<appPlatform>:<appVersion>.`);
        }
        if (!NativePlatform_1.supportedNativePlatforms.includes(platform)) {
            throw new Error(`${platform} is not a supported platform.
Supported platforms : ${NativePlatform_1.supportedNativePlatforms.join(' || ')}`);
        }
        return new AppVersionDescriptor(name, platform, version);
    }
    constructor(name, platform, version) {
        this.name = name;
        this.platform = platform;
        this.version = version;
    }
    toAppPlatformDescriptor() {
        return new AppPlatformDescriptor_1.AppPlatformDescriptor(this.name, this.platform);
    }
    toAppNameDescriptor() {
        return new AppNameDescriptor_1.AppNameDescriptor(this.name);
    }
    toString() {
        return `${this.name}:${this.platform}:${this.version}`;
    }
}
exports.AppVersionDescriptor = AppVersionDescriptor;
//# sourceMappingURL=AppVersionDescriptor.js.map