"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.launchApplication = exports.installApplicationOnSimulator = exports.runIosApp = exports.launchSimulator = exports.killAllRunningSimulators = exports.parseIOSDevicesList = exports.askUserToSelectAniPhoneSimulator = exports.askUserToSelectAniPhoneDevice = exports.getiPhoneRealDevices = exports.getComputerName = exports.getKnownDevices = exports.getiPhoneSimulators = void 0;
const lodash_1 = __importDefault(require("lodash"));
const inquirer_1 = __importDefault(require("inquirer"));
const child_process_1 = require("child_process");
const childProcess_1 = require("./childProcess");
const config_1 = __importDefault(require("./config"));
const deviceConfigUtil = __importStar(require("./deviceConfig"));
const log_1 = __importDefault(require("./log"));
const os_1 = __importDefault(require("os"));
const kax_1 = __importDefault(require("./kax"));
const node_simctl_1 = __importDefault(require("node-simctl"));
async function getiPhoneSimulators() {
    const simctl = new node_simctl_1.default();
    const iosSims = await simctl.getDevices();
    return lodash_1.default.filter(lodash_1.default.flattenDeep(lodash_1.default.map(iosSims, (val, key) => val)), (device) => device.name.match(/^iPhone|iPad/));
}
exports.getiPhoneSimulators = getiPhoneSimulators;
function getKnownDevices() {
    return (0, child_process_1.execSync)('xcrun xctrace list devices', { encoding: 'utf8' });
}
exports.getKnownDevices = getKnownDevices;
function getComputerName() {
    return os_1.default.hostname();
}
exports.getComputerName = getComputerName;
function getiPhoneRealDevices() {
    const devices = getKnownDevices();
    const computerName = getComputerName();
    return parseIOSDevicesList(devices, computerName);
}
exports.getiPhoneRealDevices = getiPhoneRealDevices;
async function askUserToSelectAniPhoneDevice(devices) {
    const choices = lodash_1.default.map(devices, (val, key) => ({
        name: `${val.name} (${val.sdk}) [${val.udid}]`,
        value: val,
    }));
    const { selectedDevice } = await inquirer_1.default.prompt([
        {
            choices,
            message: 'Choose an iOS device',
            name: 'selectedDevice',
            type: 'list',
        },
    ]);
    return selectedDevice;
}
exports.askUserToSelectAniPhoneDevice = askUserToSelectAniPhoneDevice;
async function askUserToSelectAniPhoneSimulator() {
    const simulators = await getiPhoneSimulators();
    const choices = lodash_1.default.map(simulators, (val, key) => ({
        name: `${val.name} (${val.sdk}) [${val.udid}]`,
        value: val,
    }));
    // Check if user has set the usePreviousEmulator flag to true
    const deviceConfig = config_1.default.get(deviceConfigUtil.IOS_DEVICE_CONFIG, {});
    if (choices && deviceConfig) {
        if (deviceConfig.usePreviousDevice) {
            // Get the name of previously used simulator
            const deviceUdid = deviceConfig.deviceId;
            // Check if simulator still exists
            let previousDevice;
            choices.forEach((val) => {
                if (val && val.value.udid === deviceUdid) {
                    previousDevice = val.value;
                }
            });
            if (previousDevice) {
                return previousDevice;
            }
        }
    }
    // if simulator is still not resolved
    const { selectedSimulator } = await inquirer_1.default.prompt([
        {
            choices,
            message: 'Choose an iOS simulator',
            name: 'selectedSimulator',
            type: 'list',
        },
    ]);
    // Update the emulatorConfig
    deviceConfig.deviceId = selectedSimulator.udid;
    config_1.default.set(deviceConfigUtil.IOS_DEVICE_CONFIG, deviceConfig);
    return selectedSimulator;
}
exports.askUserToSelectAniPhoneSimulator = askUserToSelectAniPhoneSimulator;
function parseIOSDevicesList(text, computerName) {
    const name = computerName.split('\n')[0];
    const devicePattern = /(.*?) \((.*?)\) \((.*?)\)/;
    return text
        .split('== Simulators ==')[0]
        .split('\n')
        .reduce((list, line) => {
        const device = line.match(devicePattern);
        if (device && !line.includes(name)) {
            list.push({
                name: device[1],
                sdk: device[2],
                udid: device[3],
            });
        }
        return list;
    }, []);
}
exports.parseIOSDevicesList = parseIOSDevicesList;
function killAllRunningSimulators() {
    try {
        (0, child_process_1.execSync)(`killall "Simulator"`);
    }
    catch (e) {
        // do nothing if there is no simulator launched
    }
}
exports.killAllRunningSimulators = killAllRunningSimulators;
async function launchSimulator(deviceUdid) {
    return new Promise((resolve, reject) => {
        const simProc = (0, child_process_1.spawn)('open', [
            '-a',
            'Simulator',
            '--args',
            '-CurrentDeviceUDID',
            deviceUdid,
        ]);
        simProc.stdout.on('data', (data) => {
            log_1.default.debug(data.toString());
        });
        simProc.stderr.on('data', (data) => {
            log_1.default.debug(data.toString());
        });
        simProc.on('close', (code) => {
            code === 0
                ? resolve()
                : reject(new Error(`xcrun failed with exit code ${code}`));
        });
    });
}
exports.launchSimulator = launchSimulator;
async function runIosApp({ appPath, bundleId, launchArgs, launchEnvVars, }) {
    const iPhoneDevice = await askUserToSelectAniPhoneSimulator();
    killAllRunningSimulators();
    await kax_1.default
        .task('Waiting for device to boot')
        .run(launchSimulator(iPhoneDevice.udid));
    await kax_1.default
        .task('Installing application on simulator')
        .run(installApplicationOnSimulator(iPhoneDevice.udid, appPath));
    await kax_1.default.task('Launching application').run(launchApplication(iPhoneDevice.udid, bundleId, {
        launchArgs,
        launchEnvVars,
    }));
}
exports.runIosApp = runIosApp;
async function installApplicationOnSimulator(deviceUdid, pathToAppFile) {
    const simctl = new node_simctl_1.default({
        udid: deviceUdid,
    });
    return simctl.installApp(pathToAppFile);
}
exports.installApplicationOnSimulator = installApplicationOnSimulator;
async function launchApplication(deviceUdid, bundleId, { launchArgs = '', launchEnvVars = '', } = {}) {
    // Prefix all provided env variables with 'SIMCTL_CHILD_
    // As described by running `xcrun simctl launch --help` :
    //    "If you want to set environment variables in the resulting environment,
    //     set them in the calling environment with a SIMCTL_CHILD_ prefix."
    launchEnvVars
        .split(' ')
        .map((x) => x.split('='))
        .forEach(([k, v]) => {
        process.env[`SIMCTL_CHILD_${k}`] = v;
    });
    return (0, childProcess_1.spawnp)('xcrun', [
        'simctl',
        'launch',
        deviceUdid,
        bundleId,
        ...launchArgs.split(' '),
    ]);
}
exports.launchApplication = launchApplication;
//# sourceMappingURL=ios.js.map