"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getReactNativeCodegenVersion = exports.getDefaultIosDeploymentTarget = exports.fillProjectHull = void 0;
const PackagePath_1 = require("./PackagePath");
const mustacheUtils = __importStar(require("./mustacheUtils"));
const mustache_1 = __importDefault(require("mustache"));
const shell_1 = __importDefault(require("./shell"));
const Manifest_1 = require("./Manifest");
const handleCopyDirective_1 = __importDefault(require("./handleCopyDirective"));
const log_1 = __importDefault(require("./log"));
const utils_1 = require("./utils");
const ModuleTypes_1 = require("./ModuleTypes");
const path_1 = __importDefault(require("path"));
const xcode_ern_1 = __importDefault(require("xcode-ern"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const lodash_1 = __importDefault(require("lodash"));
const kax_1 = __importDefault(require("./kax"));
const packageJsonFileUtils_1 = require("./packageJsonFileUtils");
const nativeDependenciesLookup_1 = require("./nativeDependenciesLookup");
const gitApply_1 = require("./gitApply");
const semver_1 = __importDefault(require("semver"));
const readDir = require("fs-readdir-recursive");
const glob_1 = __importDefault(require("glob"));
async function fillProjectHull(pathSpec, projectSpec, plugins, mustacheView, composite, iosConfig) {
    var _a, _b, _c;
    log_1.default.debug(`[=== Starting iOS framework project hull filling ===]`);
    shell_1.default.pushd(pathSpec.rootDir);
    try {
        await fs_extra_1.default.ensureDir(pathSpec.outputDir);
        shell_1.default.cp('-R', pathSpec.projectHullDir, pathSpec.outputDir);
        if (mustacheView) {
            log_1.default.debug(`Applying mustache template to Container`);
            const a = path_1.default.join(pathSpec.outputDir, 'ElectrodeContainer');
            const b = path_1.default.join(pathSpec.outputDir, 'Config');
            const files = [
                ...readDir(a).map((x) => path_1.default.join(a, x)),
                ...readDir(b).map((x) => path_1.default.join(b, x)),
            ];
            for (const file of files) {
                if (file.endsWith('.h') ||
                    file.endsWith('.m') ||
                    file.endsWith('.xcconfig')) {
                    await mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(file, mustacheView, file);
                }
            }
        }
        const projectPath = path_1.default.join(pathSpec.outputDir, `${projectSpec.projectName}.xcodeproj`, 'project.pbxproj');
        const librariesPath = path_1.default.join(pathSpec.outputDir, projectSpec.projectName, 'Libraries');
        const iosProject = await getIosProject(projectPath);
        const target = iosProject.findTargetKey(projectSpec.projectName);
        const injectPluginsTaskMsg = 'Injecting Native Dependencies';
        const injectPluginsKaxTask = kax_1.default.task(injectPluginsTaskMsg);
        const rnVersion = (_a = plugins.find((p) => p.name === 'react-native')) === null || _a === void 0 ? void 0 : _a.version;
        const additionalPods = [];
        const additionalPodspecsSources = [];
        const additionalPodfileStatements = [];
        const destPodfilePath = path_1.default.join(pathSpec.outputDir, 'Podfile');
        for (const plugin of plugins) {
            const pluginSourcePath = composite
                ? plugin.basePath
                : await (0, nativeDependenciesLookup_1.getNativeDependencyPath)(path_1.default.resolve(pathSpec.outputDir, '..', 'node_modules'), plugin);
            if (!pluginSourcePath) {
                throw new Error(`path to ${plugin.name} not found.`);
            }
            const localPluginPackage = PackagePath_1.PackagePath.fromString(pluginSourcePath);
            if (await (0, utils_1.isDependencyOfType)(localPluginPackage, ModuleTypes_1.JS_API_IMPL)) {
                log_1.default.debug('JS api implementation identified, skipping fill hull.');
                continue;
            }
            let pluginConfig = await Manifest_1.manifest.getPluginConfig(plugin, 'ios', projectSpec.projectName);
            if (!pluginConfig) {
                continue;
            }
            log_1.default.debug(`Path to ${plugin.name} plugin : ${pluginSourcePath}`);
            if (await (0, utils_1.isDependencyPathNativeApiImpl)(pluginSourcePath)) {
                // For native api implementations, if a 'ern.pluginConfig' object
                // exists in its package.json, replace pluginConfig with this one.
                const pluginPackageJson = await (0, packageJsonFileUtils_1.readPackageJson)(pluginSourcePath);
                if (pluginPackageJson.ern.pluginConfig) {
                    pluginConfig = pluginPackageJson.ern.pluginConfig.ios;
                    pluginConfig = JSON.parse(mustache_1.default.render(JSON.stringify(pluginConfig), {
                        projectName: projectSpec.projectName,
                    }));
                    pluginConfig.requiresManualLinking = true;
                }
            }
            injectPluginsKaxTask.text = `${injectPluginsTaskMsg} [${plugin.name}]`;
            const { applyPatch, copy, extraPods, extraPodspecsSources, extraPodfileStatements, pbxproj, podfile, podspec, replaceInFile, requiresManualLinking, setBuildSettings, ignorePodSpec, } = pluginConfig;
            if (semver_1.default.lt(rnVersion, '0.61.0') || requiresManualLinking) {
                if (copy) {
                    for (const c of copy) {
                        if (switchToOldDirectoryStructure(pluginSourcePath, c.source)) {
                            log_1.default.debug(`Handling copy directive: Falling back to old directory structure for API(Backward compatibility)`);
                            c.source = path_1.default.normalize('IOS/IOS/Classes/SwaggersAPIs');
                        }
                    }
                    (0, handleCopyDirective_1.default)(pluginSourcePath, pathSpec.outputDir, copy);
                }
                if (replaceInFile) {
                    for (const r of replaceInFile) {
                        const pathToFile = path_1.default.join(pathSpec.outputDir, r.path);
                        const fileContent = await fs_extra_1.default.readFile(pathToFile, 'utf8');
                        const patchedFileContent = fileContent.replace(RegExp(r.string, 'g'), r.replaceWith);
                        await fs_extra_1.default.writeFile(pathToFile, patchedFileContent, {
                            encoding: 'utf8',
                        });
                    }
                }
                if (setBuildSettings) {
                    for (const s of setBuildSettings) {
                        const pathToPbxProj = path_1.default.join(pathSpec.outputDir, s.path);
                        // Add any missing section in the target pbxproj
                        // This is necessary for proper parsing and modification of
                        // the pbxproj with the xcode-ern library
                        xcode_ern_1.default.pbxProjFileUtils().addMissingSectionsToPbxProj(pathToPbxProj);
                        const iosProj = await getIosProject(pathToPbxProj);
                        const buildSettings = s.buildSettings instanceof Array
                            ? s.buildSettings
                            : [s.buildSettings];
                        for (const buildSettingsEntry of buildSettings) {
                            for (const buildType of buildSettingsEntry.configurations) {
                                for (const key of Object.keys(buildSettingsEntry.settings)) {
                                    iosProj.updateBuildProperty(key, buildSettingsEntry.settings[key], buildType);
                                }
                            }
                        }
                        fs_extra_1.default.writeFileSync(pathToPbxProj, iosProj.writeSync());
                    }
                }
                if (applyPatch) {
                    const { patch, root, inNodeModules } = applyPatch;
                    if (!patch) {
                        throw new Error('Missing "patch" property in "applyPatch" object');
                    }
                    if (!root && !inNodeModules) {
                        throw new Error('If "inNodeModules" is not set, "root" property must be set in "applyPatch" object');
                    }
                    const [patchFile, rootDir] = [
                        path_1.default.join(pluginConfig.path, patch),
                        inNodeModules
                            ? pluginSourcePath
                            : path_1.default.join(pathSpec.outputDir, root),
                    ];
                    await (0, gitApply_1.gitApply)({ patchFile, rootDir });
                }
                if (pbxproj) {
                    const { addEmbeddedFramework, addFile, addFramework, addFrameworkReference, addFrameworkSearchPath, addHeader, addHeaderSearchPath, addProject, addSource, addStaticLibrary, } = pbxproj;
                    if (addSource) {
                        for (const source of addSource) {
                            // Multiple source files
                            if (source.from) {
                                if (switchToOldDirectoryStructure(pluginSourcePath, source.from)) {
                                    log_1.default.debug(`Source Copy: Falling back to old directory structure for API(Backward compatibility)`);
                                    source.from = path_1.default.normalize('IOS/IOS/Classes/SwaggersAPIs/*.swift');
                                }
                                const relativeSourcePath = path_1.default.dirname(source.from);
                                const pathToSourceFiles = path_1.default.join(pluginSourcePath, relativeSourcePath);
                                const fileNames = lodash_1.default.filter(await fs_extra_1.default.readdir(pathToSourceFiles), (f) => f.endsWith(path_1.default.extname(source.from)));
                                for (const fileName of fileNames) {
                                    const fileNamePath = path_1.default.join(source.path, fileName);
                                    iosProject.addSourceFile(fileNamePath, null, iosProject.findPBXGroupKey({ name: source.group }));
                                }
                            }
                            else {
                                // Single source file
                                iosProject.addSourceFile(source.path, null, iosProject.findPBXGroupKey({ name: source.group }));
                            }
                        }
                    }
                    if (addHeader) {
                        for (const header of addHeader) {
                            // Multiple header files
                            if (header.from) {
                                if (switchToOldDirectoryStructure(pluginSourcePath, header.from)) {
                                    log_1.default.debug(`Header Copy: Falling back to old directory structure for API(Backward compatibility)`);
                                    header.from = path_1.default.normalize('IOS/IOS/Classes/SwaggersAPIs/*.swift');
                                }
                                const relativeHeaderPath = path_1.default.dirname(header.from);
                                const pathToHeaderFiles = path_1.default.join(pluginSourcePath, relativeHeaderPath);
                                const fileNames = lodash_1.default.filter(await fs_extra_1.default.readdir(pathToHeaderFiles), (f) => f.endsWith(path_1.default.extname(header.from)));
                                for (const fileName of fileNames) {
                                    const fileNamePath = path_1.default.join(header.path, fileName);
                                    iosProject.addHeaderFile(fileNamePath, { public: header.public }, iosProject.findPBXGroupKey({ name: header.group }));
                                }
                            }
                            else {
                                const headerPath = header.path;
                                iosProject.addHeaderFile(headerPath, { public: header.public }, iosProject.findPBXGroupKey({ name: header.group }));
                            }
                        }
                    }
                    if (addFile) {
                        for (const file of addFile) {
                            iosProject.addFile(file.path, iosProject.findPBXGroupKey({ name: file.group }));
                            // Add target dep in any case for now, will rework later
                            iosProject.addTargetDependency(target, [
                                `"${path_1.default.basename(file.path)}"`,
                            ]);
                        }
                    }
                    if (addFramework) {
                        for (const framework of addFramework) {
                            iosProject.addFramework(framework, {
                                customFramework: true,
                            });
                        }
                    }
                    if (addProject) {
                        for (const project of addProject) {
                            const projectAbsolutePath = path_1.default.join(librariesPath, project.path, 'project.pbxproj');
                            const options = {
                                addAsTargetDependency: project.addAsTargetDependency,
                                frameworks: project.frameworks,
                                projectAbsolutePath,
                                staticLibs: project.staticLibs,
                            };
                            try {
                                iosProject.addProject(project.path, project.group, target, options);
                            }
                            catch (e) {
                                if (e.message.includes("Cannot read property 'productReference'") &&
                                    ((_b = options.staticLibs) === null || _b === void 0 ? void 0 : _b.length) >= 1) {
                                    throw new Error(`Error when trying to inject ${plugin.name} in iOS container.
This error is typically due to a plugin misconfiguration in the manifest.
Please make sure that the following static lib(s) reference(s) (name and target) in your configuration ...
${JSON.stringify(options.staticLibs, null, 2)}
... are matching the values defined in the native module pbxproj located in ${projectAbsolutePath}.
`);
                                }
                                else {
                                    throw e;
                                }
                            }
                        }
                    }
                    if (addStaticLibrary) {
                        for (const lib of addStaticLibrary) {
                            iosProject.addStaticLibrary(lib);
                        }
                    }
                    if (addHeaderSearchPath) {
                        for (const p of addHeaderSearchPath) {
                            iosProject.addToHeaderSearchPaths(p);
                        }
                    }
                    if (addFrameworkReference) {
                        for (const frameworkReference of addFrameworkReference) {
                            iosProject.addFramework(frameworkReference, {
                                customFramework: true,
                            });
                        }
                    }
                    if (addEmbeddedFramework) {
                        for (const framework of addEmbeddedFramework) {
                            iosProject.addFramework(framework, {
                                customFramework: true,
                                embed: true,
                            });
                        }
                    }
                    if (addFrameworkSearchPath) {
                        for (const p of addFrameworkSearchPath) {
                            iosProject.addToFrameworkSearchPaths(p);
                        }
                    }
                }
            }
            if (semver_1.default.gte(rnVersion, '0.61.0')) {
                const re = /.+node_modules\/(.+)/;
                const reExec = re.exec(pluginSourcePath);
                if (podfile) {
                    if (plugin.name !== 'react-native') {
                        throw new Error(`ios.podfile directive can only be used for react-native configuration`);
                    }
                    const sourcePodfilePath = path_1.default.join(pluginConfig.path, podfile);
                    shell_1.default.cp(sourcePodfilePath, destPodfilePath);
                }
                if (podspec) {
                    const sourcePodspecPath = path_1.default.join(pluginConfig.path, podspec);
                    let destPodspecPath;
                    if (reExec) {
                        destPodspecPath = path_1.default.join(pathSpec.outputDir, 'node_modules', reExec[1], podspec);
                    }
                    else {
                        destPodspecPath = path_1.default.join(pluginSourcePath, podspec);
                    }
                    shell_1.default.cp(sourcePodspecPath, destPodspecPath);
                }
                if ((ignorePodSpec && reExec) ||
                    (reExec && ((_c = iosConfig === null || iosConfig === void 0 ? void 0 : iosConfig.excludedNativeModules) === null || _c === void 0 ? void 0 : _c.includes(reExec[1])))) {
                    log_1.default.debug(`Ignoring ${reExec[1]} native module pod ${ignorePodSpec
                        ? '(ignorePodSpec directive set in manifest for this module)'
                        : '(Present in ios container config excludedNativeModules array)'}`);
                    const podspecs = await new Promise((resolve, reject) => {
                        (0, glob_1.default)(path_1.default.join(pathSpec.outputDir, 'node_modules', reExec[1], '**/*.podspec'), (err, files) => {
                            if (err) {
                                reject(err);
                            }
                            else {
                                resolve(files.map(path_1.default.normalize));
                            }
                        });
                    });
                    for (const podspecFile of podspecs) {
                        fs_extra_1.default.unlinkSync(podspecFile);
                    }
                }
                if (extraPods) {
                    additionalPods.push(...extraPods);
                }
                if (extraPodspecsSources) {
                    additionalPodspecsSources.push(...extraPodspecsSources);
                }
                if (extraPodfileStatements) {
                    additionalPodfileStatements.push(...extraPodfileStatements);
                }
            }
        }
        if (semver_1.default.gte(rnVersion, '0.61.0')) {
            // Dedupe additional specs sources and add CocoaPods master spec repository
            const finalPodspecsSources = lodash_1.default.uniq(additionalPodspecsSources.map((s) => `source '${s}'`));
            // Dedupe extra Podfile statements
            const finalExtraPodfileStatements = lodash_1.default.uniq(additionalPodfileStatements);
            // Add master pod spec repository only if there is at least a custom pod spec source
            if (finalPodspecsSources.length > 0) {
                finalPodspecsSources.push(`source 'https://github.com/CocoaPods/Specs.git'`);
            }
            await mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(destPodfilePath, {
                extraPodfileStatements: finalExtraPodfileStatements,
                extraPods: additionalPods,
                extraPodspecsSources: finalPodspecsSources,
                iosDeploymentTarget: projectSpec.deploymentTarget,
                nodeModulesRelativePath: projectSpec.nodeModulesRelativePath || './node_modules',
                projectName: projectSpec.projectName,
            }, destPodfilePath);
        }
        injectPluginsKaxTask.succeed(injectPluginsTaskMsg);
        log_1.default.debug(`[=== Completed framework generation ===]`);
        return { iosProject, projectPath };
    }
    finally {
        shell_1.default.popd();
    }
}
exports.fillProjectHull = fillProjectHull;
async function getIosProject(projectPath) {
    const project = xcode_ern_1.default.project(projectPath);
    return new Promise((resolve, reject) => {
        project.parse((err) => {
            if (err) {
                log_1.default.error(`failed to get ios project : ${JSON.stringify(err)}`);
                reject(err);
            }
            resolve(project);
        });
    });
}
function switchToOldDirectoryStructure(pluginSourcePath, tail) {
    // This is to check if the api referenced during container generation is created using the old or new directory structure to help keep the backward compatibility.
    const pathToSwaggersAPIs = path_1.default.normalize('IOS/IOS/Classes/SwaggersAPIs');
    if (path_1.default.dirname(tail) === `IOS` &&
        fs_extra_1.default.pathExistsSync(path_1.default.join(pluginSourcePath, path_1.default.dirname(pathToSwaggersAPIs)))) {
        return true;
    }
    return false;
}
const getDefaultIosDeploymentTarget = (rnVersion) => {
    if (semver_1.default.gte(rnVersion, '0.64.0')) {
        return '11.0';
    }
    else if (semver_1.default.gte(rnVersion, '0.63.0')) {
        return '10.0';
    }
    else if (semver_1.default.gte(rnVersion, '0.56.0')) {
        return '9.0';
    }
    else {
        return '8.0';
    }
};
exports.getDefaultIosDeploymentTarget = getDefaultIosDeploymentTarget;
const getReactNativeCodegenVersion = (rnVersion) => {
    if (semver_1.default.gte(rnVersion, '0.68.4')) {
        return '^0.0.18';
    }
    else if (semver_1.default.gte(rnVersion, '0.68.2')) {
        return '^0.0.17';
    }
    else if (semver_1.default.gte(rnVersion, '0.68.0')) {
        return '^0.0.13';
    }
    else if (semver_1.default.gte(rnVersion, '0.67.0')) {
        return '^0.0.8';
    }
    else if (semver_1.default.gte(rnVersion, '0.65.0')) {
        return '^0.0.7';
    }
    else if (semver_1.default.gte(rnVersion, '0.64.0')) {
        return '^0.0.6';
    }
    else {
        return null;
    }
};
exports.getReactNativeCodegenVersion = getReactNativeCodegenVersion;
//# sourceMappingURL=iosUtil.js.map