"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mustacheRenderToOutputFileUsingTemplateFile = exports.mustacheRenderUsingTemplateFile = void 0;
const mustache_1 = __importDefault(require("mustache"));
const fs_extra_1 = __importDefault(require("fs-extra"));
// =============================================================================
// Mustache related utilities
// =============================================================================
// Mustache render using a template file
// filename: Path to the template file
// view: Mustache view to apply to the template
// returns: Rendered string output
async function mustacheRenderUsingTemplateFile(filename, view, partials) {
    return fs_extra_1.default
        .readFile(filename, 'utf8')
        .then((template) => mustache_1.default.render(template, view, partials));
}
exports.mustacheRenderUsingTemplateFile = mustacheRenderUsingTemplateFile;
// Mustache render to an output file using a template file
// templateFilename: Path to the template file
// view: Mustache view to apply to the template
// outputFile: Path to the output file
async function mustacheRenderToOutputFileUsingTemplateFile(templateFilename, view, outputFile, partials) {
    return mustacheRenderUsingTemplateFile(templateFilename, view, partials).then((output) => {
        return fs_extra_1.default.writeFile(outputFile, output);
    });
}
exports.mustacheRenderToOutputFileUsingTemplateFile = mustacheRenderToOutputFileUsingTemplateFile;
//# sourceMappingURL=mustacheUtils.js.map