"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNativeDependencyPath = exports.findNativeDependencies = exports.resolvePackagePaths = exports.getUnprefixedVersion = exports.filterDirectories = exports.findDirectoriesContainingNativeCode = void 0;
const PackagePath_1 = require("./PackagePath");
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const Manifest_1 = require("./Manifest");
const ModuleTypes = __importStar(require("./ModuleTypes"));
const readDir = require("fs-readdir-recursive");
const NodeModulesLen = 'node_modules'.length;
function findDirectoriesContainingNativeCode(rootDir) {
    return readDir(rootDir)
        .filter((a) => /\.swift$|\.m$|\.pbxproj$|\.java$|\.framework\//.test(a))
        .filter((a) => !/ElectrodeContainer.framework/.test(a));
}
exports.findDirectoriesContainingNativeCode = findDirectoriesContainingNativeCode;
function filterDirectories(directories) {
    return lodash_1.default.filter(directories, (d) => !/sample|demo|example|appium|safari-launcher/i.test(d));
}
exports.filterDirectories = filterDirectories;
function getUnprefixedVersion(version) {
    return (version === null || version === void 0 ? void 0 : version.startsWith('v')) ? version.slice(1) : version;
}
exports.getUnprefixedVersion = getUnprefixedVersion;
function resolvePackagePaths(paths) {
    const result = new Set();
    for (const d of paths) {
        const lastIdx = d.lastIndexOf('node_modules');
        if (lastIdx === -1) {
            const pathSegments = d.split(path_1.default.sep);
            if (d.startsWith('@')) {
                // ex : @scoped-pkgs/pkg-native/src/code.swift
                // should return : @scoped-pkgs/pkg-native
                const p = path_1.default.join(pathSegments[0], pathSegments[1]);
                result.add(p);
            }
            else {
                // ex : pkg-native/src/code.swift
                // should return : pkg-native
                result.add(pathSegments[0]);
            }
        }
        else {
            const tmp = d.slice(lastIdx + NodeModulesLen + 1);
            const pathSegments = tmp.split(path_1.default.sep);
            if (tmp.startsWith('@')) {
                // ex : @scoped-pkgs/nested/node_modules/@scope/pkg-native/src/code.swift
                // should return : @scoped-pkgs/nested/node_modules/@scope/pkg-native
                const p = path_1.default.join(d.substring(0, lastIdx + NodeModulesLen + 1), pathSegments[0], pathSegments[1]);
                result.add(p);
            }
            else {
                // ex : @scoped-pkgs/nested/node_modules/pkg-native/src/code.swift
                // should return : @scoped-pkgs/nested/node_modules/pkg-native
                const p = path_1.default.join(d.substring(0, lastIdx + NodeModulesLen + 1), pathSegments[0]);
                result.add(p);
            }
        }
    }
    return result;
}
exports.resolvePackagePaths = resolvePackagePaths;
async function findNativeDependencies(dir, { manifestId } = {}) {
    const dirs = Array.isArray(dir) ? dir : [dir];
    const result = {
        all: [],
        apis: [],
        nativeApisImpl: [],
        thirdPartyInManifest: [],
        thirdPartyNotInManifest: [],
    };
    for (const d of dirs) {
        const directoriesWithNativeCode = findDirectoriesContainingNativeCode(d);
        const filteredDirectories = filterDirectories(directoriesWithNativeCode);
        const packagePaths = resolvePackagePaths(filteredDirectories);
        for (const packagePath of packagePaths) {
            const pathToPackage = path_1.default.join(d, packagePath);
            const pathToNativeDependencyPackageJson = path_1.default.join(pathToPackage, 'package.json');
            const nativeDepPackageJson = require(pathToNativeDependencyPackageJson);
            const dep = PackagePath_1.PackagePath.fromString(pathToPackage);
            if (nativeDepPackageJson.ern) {
                if (nativeDepPackageJson.ern.moduleType === ModuleTypes.API ||
                    /react-native-.+-api$/.test(dep.name)) {
                    result.apis.push(dep);
                }
                else if (nativeDepPackageJson.ern.moduleType === ModuleTypes.NATIVE_API_IMPL) {
                    result.nativeApisImpl.push(dep);
                }
            }
            else {
                if (await Manifest_1.manifest.getNativeDependency(dep, { manifestId })) {
                    result.thirdPartyInManifest.push(dep);
                }
                else {
                    result.thirdPartyNotInManifest.push(dep);
                }
            }
            result.all.push(dep);
        }
    }
    return result;
}
exports.findNativeDependencies = findNativeDependencies;
async function getNativeDependencyPath(dir, d) {
    const dependencies = await findNativeDependencies(dir);
    const dependency = dependencies.all.find((x) => x.name === d.name);
    return dependency === null || dependency === void 0 ? void 0 : dependency.basePath;
}
exports.getNativeDependencyPath = getNativeDependencyPath;
//# sourceMappingURL=nativeDependenciesLookup.js.map