"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPathToPackageJson = exports.writePackageJsonSync = exports.writePackageJson = exports.readPackageJsonSync = exports.readPackageJson = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const PACKAGE_JSON_FILENAME = 'package.json';
/**
 * Asynchronously reads a package.json file from a given directory
 * and returns the parsed object
 * @param p Path to the directory containing the package.json file
 */
const readPackageJson = async (p) => fs_extra_1.default.readJson(getPathToPackageJson(p));
exports.readPackageJson = readPackageJson;
/**
 * Synchronously reads a package.json file from a given directory
 * and returns the parsed object
 * @param p Path to the directory containing the package.json file
 */
const readPackageJsonSync = (p) => fs_extra_1.default.readJsonSync(getPathToPackageJson(p));
exports.readPackageJsonSync = readPackageJsonSync;
/**
 * Asynchronously writes a package.json file to a given directory and
 * returns the full path to the file
 * @param p Path to the directory where to store the package.json file
 * @param content Content of package.json as an object
 */
const writePackageJson = (p, content) => fs_extra_1.default.writeJson(getPathToPackageJson(p), content, { spaces: 2 });
exports.writePackageJson = writePackageJson;
/**
 * Synchronously writes a package.json file to a given directory and
 * returns the full path to the file
 * @param p Path to the directory where to store the package.json file
 * @param content Content of package.json as an object
 */
const writePackageJsonSync = (p, content) => fs_extra_1.default.writeJsonSync(getPathToPackageJson(p), content, { spaces: 2 });
exports.writePackageJsonSync = writePackageJsonSync;
/**
 * Given a directory holding a package.json file, return the full path
 * to the package.json file
 * @param p Path to the directory containing the package.json
 */
function getPathToPackageJson(p) {
    if (!fs_extra_1.default.statSync(p).isDirectory) {
        throw new Error(`${p} is not a directory`);
    }
    return path_1.default.join(p, PACKAGE_JSON_FILENAME);
}
exports.getPathToPackageJson = getPathToPackageJson;
//# sourceMappingURL=packageJsonFileUtils.js.map