"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.unzip = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const shelljs_1 = __importDefault(require("shelljs"));
const yauzl_1 = __importDefault(require("yauzl"));
async function unzip(zippedData, destPath) {
    await fs_extra_1.default.ensureDir(destPath);
    return new Promise((resolve, reject) => {
        yauzl_1.default.fromBuffer(zippedData, { lazyEntries: true }, (err, zipfile) => {
            if (err) {
                reject(err);
            }
            else if (zipfile == null) {
                reject(new Error('zipFile is null or undefined'));
            }
            else {
                zipfile.readEntry();
                zipfile.on('end', () => resolve());
                zipfile.on('entry', (entry) => {
                    if (/\$/.test(entry.fileName)) {
                        // Current entry is an empty directory
                        shelljs_1.default.mkdir('-p', path_1.default.join(destPath, entry.fileName));
                        zipfile.readEntry();
                    }
                    else {
                        // Current entry is a file
                        shelljs_1.default.mkdir('-p', path_1.default.join(destPath, path_1.default.dirname(entry.fileName)));
                        const ws = fs_extra_1.default.createWriteStream(path_1.default.join(destPath, entry.fileName));
                        zipfile.openReadStream(entry, (error, rs) => {
                            if (error) {
                                reject(error);
                            }
                            else if (rs == null) {
                                reject(new Error('rs is null or undefined'));
                            }
                            else {
                                rs.pipe(ws);
                                rs.on('end', () => zipfile.readEntry());
                            }
                        });
                    }
                });
            }
        });
    });
}
exports.unzip = unzip;
//# sourceMappingURL=unzip.js.map